pro bin_noaa_monthlymean
;=====================================================================
;@***h*  MW_Sounder_FCDR/bin_noaa_monthlymean.pro
;
; NAME
;        bin_noaa_monthlymean.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the specified polar orbiter and AMSU channel and range
;              of months, AMSU values are averaged on a 2.5 degree grid.
;              Monthly gridded values are written to ASCII files for later
;              use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:     
;              Daily values of AMSU are read from the files:
;
;                 $InPath/{noaa}/YYYY/amsu_{noaa}_YYYY.DDD_ch5-12.dat
;
; OUTPUTS:    
;              Monthly gridded means are written to the files:
;
;                 $OutPath/{noaa}/bin_{noaa}_YYYYMM_chan{chan}.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                noaa = The name of the NOAA satellite to process.
;                           values = {'noaa15', 'noaa16', 'noaa18', 
;                                     'noaa19', 'noaam2'}
;                chan = AMSU channel to process.
;                           values = {9, 7}
;   YEAR_BEG,JDAY_BEG = The begining year and Julian day to process
;   YEAR_END,JDAY_END = The ending year and Julian day to process
;   InPath            = The root path to the NOAA AMSU Brightness Temps files.
;   OutPath           = The root path for the gridded ASCII files to be generated.
;
;                       Path values should contain a trailing '/'
;
;**************************************************************************
     noaa='noaa15'
     chan=9
 YEAR_BEG=2006
MONTH_BEG=11
 YEAR_END=2006
MONTH_END=12
   InPath='../Test-Data/ASCII_tmp/extract/'
  OutPath='../Test-Data/ASCII_tmp/bin/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Loop over the years to process
;-----------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the starting ans ending month for the current year
  ;-------------------------------------------------------
  monthstart=1
  monthend=12
  if(year_loop eq YEAR_BEG) then monthstart=MONTH_BEG
  if(year_loop eq YEAR_END) then monthend=MONTH_END

  ; Set the Julian day for the begining of each month
  ; and the number of days in the month
  ;--------------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
    month_start=[ 1,32,61,92,122,153,183,214,245,275,306,336]
    month_long= [31,29,31,30, 31, 30, 31, 31, 30, 31, 30, 31]
  endif else begin
    month_start=[ 1,32,60,91,121,152,182,213,244,274,305,335]
    month_long= [31,28,31,30, 31, 30, 31, 31, 30, 31, 30, 31]
  endelse

  ; Loop over months
  ;-------------------------
  for month_loop=monthstart,monthend do begin

    ii_wetrt=0L

    num=20000000L
    dd_noaa  =intarr(num)
    time_noaa=intarr(num)
    iscan    =fltarr(num)
    ispot    =fltarr(num)
    lat_noaa =fltarr(num)
    lon_noaa =fltarr(num)
    sza_noaa =fltarr(num)  ; solar zenith angle
    sat_noaa =fltarr(num)  ; satellite zenith angle
    bt_noaa  =fltarr(num)  ; channels of interest

    ; Loop over the days in the current month
    ;------------------------------------------
    jbeg=month_start[month_loop-1]
    jend=month_start[month_loop-1]+month_long[month_loop-1]-1
    total_num=0L
    for day_loop=jbeg,jend do begin

      print,'day=',day_loop

      ; Construct the input file name and open it
      ;--------------------------------------------
      infile_noaa=InPath+noaa+'/'+string(year_loop,format='(i4.4)') $
               +'/amsu_'+noaa+'_'+string(year_loop,format='(i4.4)') $
                             +'.'+string( day_loop,format='(i3.3)')+'_ch5-12.dat'
      print,'infile=',infile_noaa
      openr,6,infile_noaa,ERROR=err

      ; if there was an error, goto next day
      ;--------------------------------------
      if(err ne 0) then begin
        print,' Input open error: err=',err
        goto,NEXT_DAY
      endif

      ; Get the number of records in the file.
      ; if there are none, then goto next day
      ;----------------------------------------
      inum2=long(file_lines(infile_noaa))
      print,"NOAA inum2=",inum2
      if(inum2 le 2) then begin
        print,'there is no noaa data in',day_loop
        close,6
        goto,NEXT_DAY 
      endif

      ; read in NOAA AMSU data and close the file.
      ;-------------------------------------------
      temp = fltarr(22,inum2)
      readf,6,temp
      close,6

      ; restrict indicies to only include valid temperatures
      ;-------------------------------------------------------
      valid_id=where((temp[chan+3,*] gt 0),valid_num)
      print,"NOAA inum2=",inum2, '  valid num = ',valid_num
      if(valid_num eq 0) then begin
        print,'no valid noaa data today',infile_noaa
        goto,NEXT_DAY
      endif

      ; extract the desired values
      ;-----------------------------
      dd_noaa  [total_num:total_num+valid_num-1]=temp[0,valid_id]
      time_noaa[total_num:total_num+valid_num-1]=temp[1,valid_id]
      lat_noaa [total_num:total_num+valid_num-1]=temp[4,valid_id]
      lon_noaa [total_num:total_num+valid_num-1]=temp[5,valid_id]  
      sza_noaa [total_num:total_num+valid_num-1]=temp[6,valid_id]   ; solar zenith angle
      sat_noaa [total_num:total_num+valid_num-1]=temp[7,valid_id]   ; satellite zenith angle
      bt_noaa  [total_num:total_num+valid_num-1]=temp[chan+3,valid_id] ; selected chan  

      ; increment total_num by the # of input records
      ;----------------------------------------------
      total_num=total_num+valid_num
      print,'total_num=',total_num

      ; goto target for next day
      ;---------------------------
      NEXT_DAY:
    endfor ; day_loop=jbeg,jend 

    ; If there is no data, goto next month
    ;--------------------------------------
    if(total_num eq 0) then goto,NEXT_MONTH

    ; trim the arry sizes
    ;-----------------------
    dd_noaa  =  dd_noaa[0L:total_num-1L]
    time_noaa=time_noaa[0L:total_num-1L] 
    lat_noaa = lat_noaa[0L:total_num-1L]
    lon_noaa = lon_noaa[0L:total_num-1L]  
    sza_noaa = sza_noaa[0L:total_num-1L] ; solar zenith angle
    sat_noaa = sat_noaa[0L:total_num-1L] ; satellite zenith angle
    bt_noaa  =  bt_noaa[0L:total_num-1L] ; selected chan 

    ; Construct output file name and open it.
    ;------------------------------------------
    outfile=OutPath+noaa+'/bin_'+noaa+'_'               $
                   +string( year_loop,format='(i4)'  )  $
                   +string(month_loop,format='(i2.2)')  $
                   +'_chan'+strcompress(string(chan),/remove_all)+'.dat'
    print,outfile
    err=file_test(OutPath,/DIRECTORY,/WRITE)
    if(err ne 1) then begin
      print,' OutPath does not exist or does not have write permission'
      print,' OutPath=',OutPath
      STOP
    endif
    outdir=file_dirname(outfile)
    err=file_test(outdir,/DIRECTORY,/WRITE)
    if(err ne 1 ) then begin
      file_mkdir,outdir
    endif
    openw,8,outfile,ERROR=err
    if(err ne 0) then begin
      print,' ERROR opening output file: err=',err
      print,' outfile=',outfile
      STOP
    endif

    ; Accumulate values at each grid point. write the mean 
    ; values and number of data values to output file.
    ;------------------------------------------------------
    bin_totalnum=0
    lat_inter=2.5
    lon_inter=2.5
    lat_bin=-90.
    while(lat_bin lt (90.-lat_inter)) do begin
      print,lat_bin
      lon_bin = -180.
      while(lon_bin lt (180.-lon_inter)) do begin
        bin_id=where((    lat_noaa  ge  lat_bin           ) and $
                     (    lat_noaa  lt (lat_bin+lat_inter)) and $
                     (    lon_noaa  ge  lon_bin           ) and $
                     (    lon_noaa  lt (lon_bin+lon_inter)) and $
                     (abs(sat_noaa) le  15.               ),bin_num)
        if(bin_num ge 1) then begin
          printf,8,(lat_bin+lat_inter/2.), (lon_bin+lon_inter/2.), $
                    mean(sza_noaa[bin_id]), mean(sat_noaa[bin_id]),$
                    mean( bt_noaa[bin_id]), bin_num,               $
          format='(2(f8.2,1x),3(f10.3,1x),i5)'
        endif
        lon_bin=(lon_bin+lon_inter)
      endwhile ; (lon_bin lt (180.-lon_inter))
      lat_bin=(lat_bin+lat_inter)
    endwhile   ; (lat_bin lt (90.-lat_inter))

    ; Close Output file
    ;-------------------
    close,8
    print,' '
    print,bin_totalnum

    ; goto target for next month
    ;----------------------------
    NEXT_MONTH:
  endfor  ; month_loop=monthstart,monthend 
endfor  ; year_loop=YEAR_BEG,YEAR_END 
;
; End Program
;----------------
end
