pro combine_amsu
;=====================================================================
;@***h*  MW_Sounder_FCDR/combine_amsu.pro
;
; NAME
;        combine_amsu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              Read in gridded data for the specified set of polar 
;              orbiters, combine data by a simple average of the data
;              from each satellite at each gridpoint, and write out 
;              the results in an ASCII file.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Gridded polar orbiter data values are read from file with 
;              the following directory structure and file format:
;
;                $InPath/bin_{noaa[*]}_YYYYMM_{chan}.dat
;
; OUTPUTS:     
;              Gridded combined polar orbiter data values are written to an 
;              ASCII file:
;
;                $OutPath/bin_YYYYMM_{chan}_noaqua.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;              noaa[] = The array of polar orbiters to use
;                           values = {'noaa15', 'noaa16', 'noaa18',
;                                     'noaa19', 'noaam2', 'aqua'}
;                 chan= String indicating the AMSU channel number and whether 
;                       the data has been converted using GPSRO data.
;                           values = {'chan9', 'chan9_converted'}
;  YEAR_BEG,MONTH_BEG = The begining year and Month to process
;  YEAR_END,MONTH_END = The ending year and Month to process
;   InPath            = The root path to the polar orbiter files.
;   OutPath           = The root path for the ASCII files containing 
;                       combined data
;
;                       Path values should contain a trailing '/'
;
;**************************************************************************
     noaa=['noaa15','noaa16','noaa18', 'noaa19', 'noaam2', 'aqua']
     chan='chan9_converted'
 YEAR_BEG=2006
MONTH_BEG=11
 YEAR_END=2006
MONTH_END=12
   InPath='../Test-Data/ASCII_tmp/bin/'
  OutPath='../Test-Data/ASCII_tmp/combine/'
;*********************************************************
; End of RUN parameters
;*********************************************************

; Set the number of polar orbiters specified
; Set the Lat/Lon resolution
;------------------------------
NSAT=n_elements(noaa)
total_num=lonarr(NSAT)
degree_inter=[2.5,2.5]

if(NSAT ne 6) then begin
  print,' Unfortunately, the output was hardwired for all 6 satellites '
  print,' this cannot be fixed witout reprocessing the dataset. Sorry. '
  STOP
endif

; Load Lat/Lon values
;----------------------
lat_num=180/degree_inter[0]
lon_num=360/degree_inter[1]
lat=fltarr(lat_num*lon_num) ;36 = 90/2.5, 72=180/2.5
lon=fltarr(lat_num*lon_num)

inum=0L 
lat_loop=-90. 
while(lat_loop le (90.-degree_inter[0])) do begin
  lon_loop=-180.
  while(lon_loop le (180.-degree_inter[1])) do begin
    lat[inum]=lat_loop+(degree_inter[0]/2)
    lon[inum]=lon_loop+(degree_inter[1]/2)
    inum=inum+1L
    lon_loop=lon_loop+degree_inter[1]
  endwhile ; (lon_loop le (180.-degree_inter[1]))
  lat_loop=lat_loop+degree_inter[0]
endwhile  ; (lat_loop le (90.-degree_inter[0]))

;
; Loop over the Years to process
;-------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the starting and ending month for the current year
  ;---------------------------------------------------------
  monthstart=1
  monthend  =12
  if(year_loop eq YEAR_BEG) then monthstart=MONTH_BEG
  if(year_loop eq YEAR_END) then monthend  =MONTH_END

  ; Loop over months for this year
  ;---------------------------------
  for month_loop=monthstart,monthend do begin

    ; Allocate space, initialize values
    ;--------------------------------------
    sza_all=fltarr(lat_num*lon_num,NSAT)-999.
    sat_all=fltarr(lat_num*lon_num,NSAT)-999.
    tb_all =fltarr(lat_num*lon_num,NSAT)-999.
    num_all=lonarr(lat_num*lon_num,NSAT)
    total_num[*]=0L
    num_all[*,*]=0L

    ; Loop over polar orbiters
    ;---------------------------
    for noaa_loop=0,(NSAT-1) do begin  

      ; Construct the file name for the polar orbiter
      ;------------------------------------------------
      infile=InPath+noaa[noaa_loop]+'/bin_'+noaa[noaa_loop]+'_' $
                            +string( year_loop,format='(i4.4)') $
                            +string(month_loop,format='(i2.2)') $
                            +'_'+chan+'.dat'
      print,infile

      ; open the satellite file, if there is 
      ; an error goto the next satellite
      ;---------------------------------------
      openr,lun2,infile,err=err,/get_lun
      if(err ne 0) then begin
        goto,NEXT_SATELLITE
      endif

      ; Get the number of lines in the file, if there are 
      ; not enough, then goto the next satellite, otherwise 
      ; allocate space
      ;-----------------------------------------------------
      nums=file_lines(infile)
      print,' nums=',nums
      if(nums eq 0) then begin
        goto,NEXT_SATELLITE
      endif
      data_all=fltarr(6,nums) 

      ; Read in data
      ;---------------
      readf,lun2,data_all
      close,lun2
      free_lun,lun2
      
      ; Store the data according to Lat/Lon
      ;-------------------------------------
      inum=0L
      lat_loop=-90.
      while(lat_loop le (90.-degree_inter[0])) do begin
        lon_loop=-180.
        while(lon_loop le (180.-degree_inter[1])) do begin

          ; get the index for the current Lat/Lon
          ;----------------------------------------
          latlon_id1=where((data_all[0,*] eq (lat_loop+degree_inter[0]/2)) and $
                           (data_all[1,*] eq (lon_loop+degree_inter[1]/2)),latlon_num1)

          ; Copy values for later use.
          ;-----------------------------
          if(latlon_num1 eq 1) then begin
            sza_all[inum,noaa_loop]=data_all[2,latlon_id1]       ;lat
            sat_all[inum,noaa_loop]=data_all[3,latlon_id1]       ;lon
            tb_all [inum,noaa_loop]=data_all[4,latlon_id1]       ;tpw_raw
            num_all[inum,noaa_loop]=long(data_all[5,latlon_id1]) ;tpw_extra
          endif

          ; Increment the data index
          ;--------------------------
          inum=inum+1L
          lon_loop=lon_loop+degree_inter[1]

        endwhile ; (lon_loop le (180.-degree_inter[1]))
        lat_loop=lat_loop+degree_inter[0]
      endwhile ; (lat_loop le (90.-degree_inter[0]))

      ; add up the total number of observations
      ;-----------------------------------------
      total_num[noaa_loop]=total(num_all[*,noaa_loop])

      ; goto target for next satellite
      ;--------------------------------
      NEXT_SATELLITE:
    endfor  ;noaa_loop
   
    ; Open output file
    ;--------------------------
    outfile=OutPath+'bin_'+string( year_loop,format='(i4.4)') $
                          +string(month_loop,format='(i2.2)') $
                          +'_'+chan+'.dat'
    openw,lun3,outfile,/get_lun

    ; Write out the number of observations for each satellite
    ;----------------------------------------------------------
    printf,lun3,noaa[0],total_num[0],noaa[1],total_num[1], $
                noaa[2],total_num[2],noaa[3],total_num[3], $
                noaa[4],total_num[4],noaa[5],total_num[5], $
    format='(6(a6,x,i9,4x))'

    ; Loop over lat-lon index, compute the mean Tb  
    ;-----------------------------------------------
    for jj=0,(lat_num*lon_num-1) do begin

      ; get the indeces for valid satellites
      ;---------------------------------------
      valid_id=where((tb_all[jj,*] gt -900),valid_num)

      ; If there is data present, calculate average values
      ; and write the reulsts to the output file
      ;-----------------------------------------------------
      if(valid_num gt 0) then begin
        sza_combine=mean( sza_all[jj,valid_id])
        sat_combine=mean( sat_all[jj,valid_id])
        tb_combine =mean(  tb_all[jj,valid_id])
        num_combine=total(num_all[jj,valid_id])
        printf,lun3,lat[jj],lon[jj],sza_combine,        $
                    sat_combine,tb_combine,num_combine, $
        format='(2(f8.2,1x),3(f10.3,1x),i5)'
      endif
    endfor ; jj=0,(lat_num*lon_num-1) 

    
    valid_id=where((tb_all[*,*] gt -900),valid_num)
    print,' T:',min(tb_all[valid_id]),max(tb_all[valid_id])
    print,' N:',min(num_all[valid_id]),max(num_all[valid_id])

    ; Close output file
    ;----------------------
    close,lun3
    free_lun,lun3

  endfor  ; month_loop=monthstart,monthend
endfor ; year_loop=YEAR_BEG,YEAR_END 
;
; End Program
;---------------
end
