pro convert_amsu_bygps
;=====================================================================
;@***h*  MW_Sounder_FCDR/convert_amsu_bygps.pro
;
; NAME
;        convert_amsu_bygps.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the specified AMSU dataset, Read in GPSRO correction
;              coeffcients from the specfied file, read in gridded AMSU
;              data, apply the correction, and write the converted data
;              to the given output file.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Monthly linear fit coefs between NOAA and GPSRO data
;              are read from:
;                $InPath_coef/timeseries_offset_slope_{noaa}_gpsro.dat
;
;              Gridded NOAA AMSU values are read from:
;                $InPath_noaa/bin_{noaa}_YYYYMM_chan9.dat
;
; OUTPUTS:     
;              Converted NOAA AMSU values are written to the 
;              ASCCI file:
;                $OutPath_noaa/bin_{noaa}_YYYYMM_chan9_converted.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;        noaa = The polar orbiter data to process.
;                 values={'noaa15', 'noaa16', 'aqua',
;                         'noaa18', 'noaa19', 'noaam2'}
; InPath_coef = Path to monthly fit coeffcients.
; InPath_noaa = Path to gridded NOAA data.
;OutPath_noaa = Path to converted NOAA data.
;
;              Path values should contain a trailing '/'
;**************************************************************************
        noaa='noaa18' 
 InPath_coef='../Test-Data/ASCII_tmp/offset_slope/'
 InPath_noaa='../Test-Data/ASCII_tmp/bin/'
OutPath_noaa='../Test-Data/ASCII_tmp/bin/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Construct the input coefficient file name
;--------------------------------------------
infile_coeff=InPath_coef+'timeseries_offset_slope_'+noaa+'_gpsro.dat'

; Open coefficient file , allocate space 
; and read in coeffcients. 
;-----------------------------------------
openr,lun2,infile_coeff,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening coeffcient file: err=',err
  print,' infile_coef=',infile_coef
  STOP
endif else begin
  num=file_lines(infile_coeff)
  temp=fltarr(7,num)
  readf,lun2,temp
  close,lun2
  free_lun,lun2
endelse
 
; Extract the desired values
;----------------------------
year      =fix(temp[0,*])
month     =fix(temp[1,*])
offset    =temp[2,*] 
slope     =temp[3,*]
sample_num=fix(temp[6,*])

; Loop over the months in the coef file
;-------------------------------------
for num_loop=0,(num-1) do begin

  ; Construct the NOAA input file name
  ;-------------------------------------
  infile=InPath_noaa+noaa+'/bin_'+noaa+'_'                   $
                    +string( year[num_loop],format='(i4.4)') $
                    +string(month[num_loop],format='(i2.2)')+'_chan9.dat'

  ; Open NOAA file, allocate space 
  ; and read in data
  ;--------------------------------
  openr,lun2,infile,ERROR=err,/get_lun
  if(err ne 0) then begin
    print,' ERROR opening input NOAA file: err=',err
    print,' infile=',infile
    STOP
  endif else begin
    bnum=file_lines(infile)
    temp=fltarr(6,bnum)
    readf,lun2,temp
    close,lun2
    free_lun,lun2
  endelse

  ; Apply the linear correction to the temperture values
  ;-----------------------------------------------------
  temp[4,*]=temp[4,*]*slope[num_loop]+offset[num_loop]

  ; Construct the converted NOAA file name
  ;----------------------------------------
  outfile=OutPath_noaa+noaa+'/bin_'+noaa+'_'                   $
                      +string( year[num_loop],format='(i4.4)') $
                      +string(month[num_loop],format='(i2.2)') $
                      +'_chan9_converted.dat'
  print,outfile
    
  ; Open output file
  ;------------------------
  openw,lun3,outfile,ERROR=err,/get_lun
  if(err ne 0) then begin
    print,' ERROR opening output NOAA file: err=',err
    print,' outfile=',outfile
    STOP
  endif

  ; Loop over lines, write out the NOAA data
  ;------------------------------------------
  for ii=0,bnum-1 do begin
    printf,lun3,temp[*,ii],format='(2(f8.2,1x),3(f10.3,1x),i5)'
  endfor

  ; Close output file
  ;---------------------
  close,lun3
  free_lun,lun3
endfor ; num_loop=0,(num-1) 
;
; End Program
;--------------
end
