pro extract_aqua_amsu
;=====================================================================
;@***h*  MW_Sounder_FCDR/extract_aqua_amsu.pro
;
; NAME
;        extract_aqua_amsu.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the AQUA polar orbiter, this program loops over the 
;              specified range of months and years, and reads the L1B 
;              HDF files from the specified input directory. 
;              (downloaded from NASA website:                            )
;              ( http://disc.sci.gsfc.nasa.gov/AIRS/data-holdings/       )
;              (                      by-data-product/amsuL1B_Rad.shtml  )
;              Values of brightness temperature and related data are stored 
;              for each julian day in ASCII files for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              The L1B HDF files should be stored with the following directory
;              structure and file format:
;
;               $InPath/aqua/YYYYMM/AIRS.YYYY.MM.DD.SSS.L1B.AMSU_Rad.v5.*.hdf
;
; OUTPUTS:     
;              Output ASCII files are stored with the following directory 
;              structure and file format:
;
;               $OutPath/aqua/YYYY/amsu_aqua_YYYY.DDD.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================

;**************************************************************************
; Set RUN parameters here:
;   YEAR_BEG,MONTH_BEG = The begining month/year to process
;   YEAR_END,MONTH_END = The ending month/year to process
;   InPath             = The root path to the AQUA L1B files.
;   OutPath            = The root path for the ASCII file to be generated.
;   OutPath            = The root path for the ASCII file to be generated.
;   MAX_SCANANG        = The maximum magnitude of scan angle to include
;
;                       Path values should contain a trailing '/'
;**************************************************************************
 YEAR_BEG=2006
MONTH_BEG=11
 YEAR_END=2006
MONTH_END=12
   InPath='../Test-Data/Input/AMSU/'
  OutPath='../Test-Data/ASCII_tmp/extract/'
MAX_SCANANG=10
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Loop over the years to process
;---------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the starting and ending month for the current year
  ;---------------------------------------------------------
  monthstart=1
  monthend=12
  if(year_loop eq YEAR_BEG) then monthstart=MONTH_BEG
  if(year_loop eq YEAR_END) then   monthend=MONTH_END

  ; Set the Julian day for the begining of each month
  ; and the number of days in the month
  ;--------------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
   month_long  =[31,29,31,30,31,30,31,31,30,31,30,31]
   month_start =[1,32,61,92,122,153,183,214,245,275,306,336]
  endif else begin
   month_long  =[31,28,31,30,31,30,31,31,30,31,30,31]
   month_start =[1,32,60,91,121,152,182,213,244,274,305,335]
  endelse

  ; Loop over months
  ;---------------------
  for month_loop=monthstart,monthend do begin

    ; Loop over the days int the current month
    ;-------------------------------------------
    jbeg=month_start[month_loop-1]
    jend=month_start[month_loop-1]+month_long[month_loop-1]-1
    for day_loop=jbeg,jend do begin
      cal_day=day_loop+1-jbeg

      ; Construct the output file names and open them
      ;---------------------------------------------------
      outfile=OutPath+'aqua/'+string(year_loop,format='(i4.4)')     $
               +'/amsu_aqua_'+string(year_loop,format='(i4.4)')+'.' $
                             +string( day_loop,format='(i3.3)')+'.dat' 
      err=file_test(OutPath,/DIRECTORY,/WRITE)
      if(err ne 1) then begin
       print,' OutPath does not exist or does not have write permission'
       print,' OutPath=',OutPath
       STOP
      endif
      outdir=file_dirname(outfile)
      err=file_test(outdir,/DIRECTORY,/WRITE)
      if(err ne 1 ) then begin
       file_mkdir,outdir
      endif
      openw,fileinfoID1,outfile,ERROR=err,/get_lun
      if(err ne 0) then begin
       print,' ERROR opening output file '
       print,' outfile=',outfile
       STOP
      endif

      ; search for HDF files for the current day
      ; file format:
      ;    AIRS.2005.01.07.177.L1B.AMSU_Rad.v5.0.0.0.G07085164138.hdf
      ;
      ; if no files are found, then goto to the next day
      ;----------------------------------------------------------
      datadir=InPath+'aqua/'+string( year_loop,format='(i4.4)')  $
                            +string(month_loop,format='(i2.2)')+'/'
      string_filename='*'+string( year_loop,format='(i4.4)')+'.' $
                         +string(month_loop,format='(i2.2)')+'.' $
                         +string(   cal_day,format='(i2.2)')+'*.hdf'
      a=file_search(datadir+string_filename,count=icount)
      printf,fileinfoID1,icount
      if(icount eq 0) then begin
        goto,NEXT_DAY
      endif

      ; Loop over the number of files found
      ;---------------------------------------
      for ik=0L,(icount-1) do begin

        ; set the filename and open it
        ; assign a fileID
        ;-----------------------------------------
        infile=a[ik]
        fileid=hdf_sd_start(infile, /read)

        ; read in data
        ;---------------
        sdsid=hdf_sd_select(fileid,0)
        HDF_SD_GetData,sdsid,lat      ;30*45
        sdsid=hdf_sd_select(fileid,1)
        HDF_SD_GetData,sdsid,lon      ;30*45
        sdsid=hdf_sd_select(fileid,2)
        HDF_SD_GetData,sdsid,time     ;30*45
        sdsid=hdf_sd_select(fileid,3)
        HDF_SD_GetData,sdsid,scanang  ;30*45
        sdsid=hdf_sd_select(fileid,9)
        HDF_SD_GetData,sdsid,solarzen ;30*45
        sdsid=hdf_sd_select(fileid,17)
        HDF_SD_GetData,sdsid,tbs      ;15*30*45
        sdsid=hdf_sd_select(fileid,18)
        HDF_SD_GetData,sdsid,tbs_err  ;15*30*45

        ; set the indices where the scan angle is small enough, 
        ; if no indices are seleceted, then goto the NEXT_FILE
        ;-------------------------------------------------------
        ang_where=where((abs(scanang) lt MAX_SCANANG),ang_count)
        if(ang_count eq 0) then goto,NEXT_FILE

        ; get AMSU brightness temperatures and related values
        ;  tbs's construction is [15*30*45]
        ;-----------------------------------------------------
        for ii=0,(ang_count-1) do begin
          ang_index=ang_where[ii]

          tbs_chanel_1=tbs[0,*,*]
          tbs_chanel_2=tbs[1,*,*]
          tbs_chanel_3=tbs[2,*,*]
          tbs_chanel_4=tbs[3,*,*]
          tbs_chanel_5=tbs[4,*,*]
          tbs_chanel_6=tbs[5,*,*]
          tbs_chanel_7=tbs[6,*,*]
          tbs_chanel_8=tbs[7,*,*]
          tbs_chanel_9=tbs[8,*,*]
          tbs_chanel_10=tbs[9,*,*]
          tbs_chanel_11=tbs[10,*,*]
          tbs_chanel_12=tbs[11,*,*]
          tbs_chanel_13=tbs[12,*,*]
          tbs_chanel_14=tbs[13,*,*]
          tbs_chanel_15=tbs[14,*,*]

          tbs_err_chanel_1=tbs_err[0,*,*]
          tbs_err_chanel_2=tbs_err[1,*,*]
          tbs_err_chanel_3=tbs_err[2,*,*]
          tbs_err_chanel_4=tbs_err[3,*,*]
          tbs_err_chanel_5=tbs_err[4,*,*]
          tbs_err_chanel_6=tbs_err[5,*,*]
          tbs_err_chanel_7=tbs_err[6,*,*]
          tbs_err_chanel_8=tbs_err[7,*,*]
          tbs_err_chanel_9=tbs_err[8,*,*]
          tbs_err_chanel_10=tbs_err[9,*,*]
          tbs_err_chanel_11=tbs_err[10,*,*]
          tbs_err_chanel_12=tbs_err[11,*,*]
          tbs_err_chanel_13=tbs_err[12,*,*]
          tbs_err_chanel_14=tbs_err[13,*,*]
          tbs_err_chanel_15=tbs_err[14,*,*]

          ; transfer the julian time to year-month-day-hour-min-sec
          ; AIRS.2006.11.13.116.L1B.AMSU_Rad.v5.0.0.0.G07222223230.hdf
          ; get the year month and day for the filename
          ;----------------------------------------------
          year =fix(strmid(infile,52,4,/REVERSE_OFFSET))
          month=fix(strmid(infile,47,2,/REVERSE_OFFSET))
          day  =fix(strmid(infile,44,2,/REVERSE_OFFSET))

          ; now get the hour min second from the julian time
          ; the beginning time is 1993.001, 0 hour,0 minute, 0 second
          ; 1996,2000,2004,2008 are leap year
          ;----------------------------------------------
          julian_time=time[ang_index]
          Jdif=(julian_time/86400.D0)-(5.5D0/1440.D0)
          Jdif=(Jdif-double(fix(Jdif)))
          Jdif=(Jdif*24.D0)
          hour=fix(Jdif)
          Jdif=((Jdif-double(hour))*60.D0)
          min =fix(Jdif)
          sec=(Jdif-double(min))*60.D0

          ; Write out AMSU brightness temperatures
          ;--------------------------------------------
           syear=string( year,format='(i4.4)')
          smonth=string(month,format='(i2.2)')
            sday=string(  day,format='(i2.2)')
          printf,fileinfoID1,syear,smonth,sday,hour,min,sec,lat[ang_index], $
                 lon[ang_index], scanang[ang_index],solarzen[ang_index],    $
                 tbs_chanel_1 [ang_index], tbs_chanel_2 [ang_index],        $
                 tbs_chanel_3 [ang_index], tbs_chanel_4 [ang_index],        $
                 tbs_chanel_5 [ang_index], tbs_chanel_6 [ang_index],        $
                 tbs_chanel_7 [ang_index], tbs_chanel_8 [ang_index],        $
                 tbs_chanel_9 [ang_index], tbs_chanel_10[ang_index],        $
                 tbs_chanel_11[ang_index], tbs_chanel_12[ang_index],        $
                 tbs_chanel_13[ang_index], tbs_chanel_14[ang_index],        $
                 tbs_chanel_15[ang_index],                                  $
          format='(1x,A4,2a3,2i3,f8.3,2(1x,f12.6),(1x,f10.5),(1x,f10.3),15(1x,f10.3))'
        endfor ; ii=0,(ang_count-1) 

        ; goto target for next file
        ;---------------------------
        NEXT_FILE:

        ; close HDF file
        ;-----------------
        hdf_sd_endaccess, sdsid
        hdf_sd_end, fileid
      endfor  ; ik=0L,(icount-1) 

      ; goto target for next day
      ;----------------------------
      NEXT_DAY:

      ; release file IDs for this day
      ;----------------------------
      close,fileinfoID1
      free_lun,fileinfoID1
    endfor ; day_loop=jbeg,jend 
  endfor ; month_loop=monthstart,monthend 
endfor  ; year_loop=YEAR_BEG,YEAR_END 
;
; End Program
;-----------------
end
