@readstd.pro
pro extract_gpsro_profiles
;=====================================================================
;@***h*  MW_Sounder_FCDR/extract_gpsro_profiles.pro
;
; NAME
;        extract_gpsro_profiles.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the specified GPSRO mission, this program loops over
;              the specified range of julian days and years, and reads 
;              in all of the WET/ATM profiles for each day.
;
;              The daily profiles DRY temperatures from 'atmPrf' files and 
;              profiles of water vapor from 'wetPrf' files are interpolated 
;              to the 100 fixed pressure levels used by the AMSU forward 
;              model. Missing data values are replaced using Standard 
;              atmosphere values.
;
;              The resulting interpolated profiles are related values for 
;              each Julian day are stored for later use.
;
; EXTERNALS:   
;              readstd.pro
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              GPSRO level2 profile files should be stored with the following 
;              directory structure and file format:
;
;                $InPath/{gps}/level2/atmPrf/YYYY.DDD/atmPrf_MMMM.YYYY.DDD.*_nc
;                $InPath/{gps}/level2/wetPrf/YYYY.DDD/wetPrf_MMMM.YYYY.DDD.*_nc
;
;               ** This program requires that matched 'atm' and 'wet' profiles 
;                  are used. To achieve this, the set of ATM files are read
;                  from the $InPath directory. Then the 2 occurances of 'atm'
;                  in each full file name are replaced with 'wet'. The resulting 
;                  WET profile must exist to be included. Because of this, 
;                  the user must be careful not to include 'atm' anywhere in the 
;                  $InPath directory name or the program will fail.
;
; OUTPUTS:     
;              Output ASCII files are stored with the following directory 
;              structure and file format:
;
;                $OutPath/{gps}/YYYY/Prf_YYYY.DDD.dat'
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;             StdPath = Path to the Standard Atmosphere files for Temperature 
;                       and Water Vapor {'stdmoist.gr', 'stdtemp.gr'}
;                 gps = The name of the GPSRO mission to process.
;                           values = {'champ', 'cosmic', 'grace', 'metopa'}
;   YEAR_BEG,JDAY_BEG = The begining year and Julian day to process
;   YEAR_END,JDAY_END = The ending year and Julian day to process
;   InPath            = The root path to the GPSRO level2 data files.
;   OutPath           = The root path for the ASCII files to be generated.
;
;                       Path values should contain a trailing '/'
;**************************************************************************
 StdPath='./Anc_Data/'
     gps='champ' 
YEAR_BEG=2006
JDAY_BEG=300
YEAR_END=2006
JDAY_END=365
  InPath='../Test-Data/Input/GPSRO/'
 OutPath='../Test-Data/ASCII_tmp/extract/'
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Set the fixed vertical levels defined in the AMSU forward model
;--- --------------------------------------------------------------------
Ps=[   0.0050,    0.0161,    0.0384,    0.0769,    0.1370,            $
       0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972, $
       1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204, $
       5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492, $
      14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829, $
      29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882, $
      51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396, $
      83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775, $
     125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784, $
     180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338, $
     247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369, $
     328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738, $
     424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200, $
     535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398, $
     661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897, $
     802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236, $
     958.5911,  986.0666, 990.9476, 995.2319, 1008.170]
nlev=n_elements(Ps)

; Set seasonal flags (monthly)
;-----------------------------
moni=[1, 1, 1, -1, -1, -1, -1, -1, -1, 1, 1, 1]

; Set MAXLEN larger than the max number of profiles in one day
;----------------------------------------------------------
MAXLEN = 5000L 

;
; Search for standard atmospheric information files
; read in temperature and moisture values.
;-----------------------------------------------------
fstdt=file_search(StdPath,'stdtemp.gr' ,count=cstdt)
fstdm=file_search(StdPath,'stdmoist.gr',count=cstdm)
if((cstdt eq 0) or (cstdm eq 0)) then begin
  print,' ERROR: Cannot find the standard atmosphere files'
  STOP
endif
readstd,fstdt,Ps,Tps
readstd,fstdm,Ps,Vps

; Allocate space for mixing Standard atmosphere mixing ratio 
; set the mix-ratio 0 from higher than 150hPa 
;--------------------------------------------------------------
Mixps=fltarr(5,100)
Mixps[0,50:99]=(622.*Vps[0,50:99]/(Ps[50:99]-Vps[0,50:99]))+1.e-10
Mixps[1,50:99]=(622.*Vps[1,50:99]/(Ps[50:99]-Vps[1,50:99]))+1.e-10
Mixps[2,50:99]=(622.*Vps[2,50:99]/(Ps[50:99]-Vps[2,50:99]))+1.e-10
Mixps[3,50:99]=(622.*Vps[3,50:99]/(Ps[50:99]-Vps[3,50:99]))+1.e-10
Mixps[4,50:99]=(622.*Vps[4,50:99]/(Ps[50:99]-Vps[4,50:99]))+1.e-10

;
; Loop over the years to process
;----------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the number of Julian days for this year
  ;--------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
    NJdays=366
  endif else begin
    NJdays=365
  endelse

  ; Set the starting and ending Julian days
  ;------------------------------------------
  day_start=1
  day_end  =NJdays
  if(year_loop eq YEAR_BEG) then day_start=JDAY_BEG
  if(year_loop eq YEAR_END) then day_end  =JDAY_END

  ; Loop over the Julian days for this year
  ;-----------------------------------------
  for day_loop=day_start,day_end do begin
    indir=InPath+gps+'/level2/atmPrf/'+string(year_loop,format='(i4.4)')+'.' $
                                      +string( day_loop,format='(i3.3)')

    ; Allocate Space
    ;--------------------
    Ts    =fltarr(MAXLEN,nlev)
    Mixs  =fltarr(MAXLEN,nlev)
    laty  =fltarr(MAXLEN)
    lonx  =fltarr(MAXLEN)
    yrz   =intarr(MAXLEN)
    monz  =intarr(MAXLEN)
    dayz  =strarr(MAXLEN)
    time  =fltarr(MAXLEN)
    gps_id=strarr(MAXLEN)
    leo_id=strarr(MAXLEN)
    irsz  =intarr(MAXLEN)

    ; Get ATM files for the current year/day
    ;----------------------------------------
    a=file_search(indir+'/atmPrf_????.'                          $
                        +string(year_loop,format='(i4.4)')+'.'   $
                        +string( day_loop,format='(i3.3)')+'*_nc',count=icount)

    in=0L
    for ik=0L,(icount-1) do begin
      infile1=a[ik]

      ; The WET profile name is the same as that fot ATM, except
      ; 2 occurances of 'atm' need to be 'wet'
      ;---------------------------------------------------------
      infile2=infile1
      strput,infile2,'wet',strpos(infile2,'atm')
      strput,infile2,'wet',strpos(infile2,'atm')

      ; verify that the WET profile exists
      ;-------------------------------------
      b=findfile(infile2,count=num)
      if(num eq 0) then begin
        print,'atm and wet do not match',infile1,infile2
        goto,NEXT_PROFILE
      endif
    
      ;-------------------------------------
      ; read the information in the atmprf
      ;-------------------------------------
      bad0_size=size(bad0)
      if(bad0_size(1) ne 0) then nouse=temporary(bad0)

      ncid = NCDF_OPEN(infile1)
      NCDF_VARGET,ncid,5,Pres
      NCDF_VARGET,ncid,6,Temp
      NCDF_ATTGET,ncid,/GLOBAL,'year'     ,year0
      NCDF_ATTGET,ncid,/GLOBAL,'month'    ,month0
      NCDF_ATTGET,ncid,/GLOBAL,'day'      ,day0
      NCDF_ATTGET,ncid,/GLOBAL,'hour'     ,hour0
      NCDF_ATTGET,ncid,/GLOBAL,'minute'   ,minute0
      NCDF_ATTGET,ncid,/GLOBAL,'second'   ,second0
      NCDF_ATTGET,ncid,/GLOBAL,'lat'      ,lat0
      NCDF_ATTGET,ncid,/GLOBAL,'lon'      ,lon0
      NCDF_ATTGET,ncid,/GLOBAL,'fileStamp',fileStamp0
      NCDF_ATTGET,ncid,/GLOBAL,'irs'      ,irs0
      NCDF_ATTGET,ncid,/GLOBAL,'bad'      ,bad0
      NCDF_CLOSE ,ncid 

      ; check the BAD flag, skip bad profiles
      ;------------------------------------------
      bad0=fix(string(bad0))
      bad0_size=size(bad0)
      if(bad0_size(1) eq 0) then goto,NEXT_PROFILE  ;bad0 is unavailable
      if(bad0         ne 0) then goto,NEXT_PROFILE

      ; save desired values
      ;------------------------------
      yrz[in]      =year0
      monz[in]     =month0
      dayz[in]     =fix(strmid(filestamp0,10,3)) 
      time[in]     =float(hour0)+float(minute0)/60.+float(second0)/3600.
      laty[in]     =float(lat0)
      lonx[in]     =float(lon0)
      gps_id[in]   =fix(strmid(fileStamp0,21,2)) 
      leo_id_string=strmid(fileStamp0,2,2) 

      if(strcmp(leo_id_string,'1') or strcmp(leo_id_string,'2') or $
         strcmp(leo_id_string,'2') or strcmp(leo_id_string,'4') or $
         strcmp(leo_id_string,'5') or strcmp(leo_id_string,'6')    )then begin
        leo_id[in]=fix(leo_id_string)   
      endif else begin
        leo_id[in]=1
      endelse
      irsz[in] = irs0
  
      ; Get indices of valid data
      ; if there are no valid data, goto next profile
      ;----------------------------------------------
      valid_range=where((Temp gt -900) and (Pres gt .0),valid_num)
      if(valid_num lt 4) then goto,NEXT_PROFILE

      tmp=where(((Ps le max(Pres[valid_range])) and $
                 (Ps ge min(Pres[valid_range]))     ),c,complement=ntmp,ncomplement=nc)
      if(c eq 0) then goto,NEXT_PROFILE  

      ; Interpolate to Ps levels, set missing values to NAN
      ;-----------------------------------------------------
      Ts[in,tmp]=interpol(Temp[valid_range],Pres[valid_range],Ps[tmp])+273.
      if(nc ge 1) then Ts[in,ntmp]=!Values.F_NaN
    

      ;-------------------------------------
      ; read the information in the wetprf
      ;-------------------------------------
      bad2_size=size(bad2)
      if(bad2_size(1) ne 0) then nouse=temporary(bad2)

      ncid=NCDF_OPEN(infile2)
      NCDF_VARGET,ncid,2,Vp
      NCDF_VARGET,ncid,3,Pres
      NCDF_ATTGET,ncid,/GLOBAL,'bad',bad2
      NCDF_CLOSE ,ncid 

      ; check the BAD flag, skip bad profiles
      ;------------------------------------------
      bad2=fix(string(bad2))
      bad2_size=size(bad2)
      if(bad2_size(1) eq 0) then goto,NEXT_PROFILE
      if(bad2         ne 0) then goto,NEXT_PROFILE

      ; Get indices of valid data
      ; if there are no valid data, goto next profile
      ;----------------------------------------------
      valid_range=where((Vp gt .0) and (Pres gt .0),valid_num)
      if(valid_num lt 4) then goto,NEXT_PROFILE

      tmp=where(((Ps le max(Pres[valid_range])) and $
                 (Ps ge min(Pres[valid_range]))   ),c,complement=ntmp,ncomplement=nc)

      ; Interpolate mixing ratio data to Ps levels, set missing values to NAN
      ;------------------------------------------------------------------------
      Mix=622.*Vp/(Pres-Vp+1.e-10)+1.e-10
      if(c  ge 1) then begin
        Mixs[in,tmp]=interpol(Mix[valid_range],Pres[valid_range],Ps[tmp])
      endif
      if(nc ge 1) then Mixs[in,ntmp]=!Values.F_NaN

      ; both ATM temps and WET mixing ratio have been 
      ; interpolated, increment the profile counter
      ;------------------------------------------------
      in=in+1

      ; goto target for next profile
      ;------------------------------
      NEXT_PROFILE:
    endfor  ;ik, all profiles in one day

    ; If the are no profiles, goto to next day
    ;--------------------------------------------
    if(in eq 0) then goto,NEXT_DAY

    ; Trim array sizes
    ;---------------------
    yrz   =   yrz[0:in-1]
    monz  =  monz[0:in-1]
    dayz  =  dayz[0:in-1]
    time  =  time[0:in-1]
    laty  =  laty[0:in-1]
    lonx  =  lonx[0:in-1]
    gps_id=gps_id[0:in-1]
    leo_id=leo_id[0:in-1]
    irsz  =  irsz[0:in-1]
    Ts    =    Ts[0:in-1,*]  ;100 levels
    Mixs  =  Mixs[0:in-1,*]  ;100 levels

    ; sort values by lat , from -90 ~ 90
    ;------------------------------------
    sort_lat=sort(laty)
    yrz   =   yrz[sort_lat]
    monz  =  monz[sort_lat]
    dayz  =  dayz[sort_lat]
    time  =  time[sort_lat]
    laty  =  laty[sort_lat]
    lonx  =  lonx[sort_lat]
    gps_id=gps_id[sort_lat]
    leo_id=leo_id[sort_lat]
    irsz  =  irsz[sort_lat]
    Ts    =    Ts[sort_lat,*]
    Mixs  =  Mixs[sort_lat,*]

    ; Allocate space for reference values
    ;---------------------------------------
    Ts0   =fltarr(in,100)
    Mix0  =fltarr(in,100)
    Valid1=intarr(in,100)+1
    valid2=intarr(in,100)+1

    ; Load reference (1) tropical (20N ~ 20S)  
    ;----------------------------------------
    tmp_lat=where((abs(laty) le 20),c11)
    for ii=0,(c11-1) do begin
       Ts0[tmp_lat[ii],*]=  Tps[0,*]
      Mix0[tmp_lat[ii],*]=Mixps[0,*]
    endfor

    ; Load reference (2) midlat (20N ~ 60N) summer
    ;--------------------------------------------------
    tmp_lat=where((abs(laty+moni[monz-1]*40) le 20),c22)
    for ii=0,(c22-1) do begin
       Ts0[tmp_lat[ii],*]=  Tps[1,*]
      Mix0[tmp_lat[ii],*]=Mixps[1,*]
    endfor

    ; Load reference (3) midlat (20N ~ 60N) winter
    ;--------------------------------------------------
    tmp_lat=where((abs(laty-moni[monz-1]*40) le 20),c3)
    for ii=0,(c3-1) do begin
       Ts0[tmp_lat[ii],*]=  Tps[2,*]
      Mix0[tmp_lat[ii],*]=Mixps[2,*]
    endfor

    ; Load reference (4) subarctic (60N ~ 90N) summer
    ;--------------------------------------------------
    tmp_lat=where((abs(laty+moni[monz-1]*75) le 15),c4)
    for ii=0,(c4-1) do begin
       Ts0[tmp_lat[ii],*]=  Tps[3,*]
      Mix0[tmp_lat[ii],*]=Mixps[3,*]
    endfor

    ; Load reference (5) subarctic (60N ~ 90N) winter
    ;--------------------------------------------------
    tmp_lat=where((abs(laty-moni[monz-1]*75) le 15),c5)
    for ii=0,(c5-1) do begin
       Ts0[tmp_lat[ii], *]=  Tps[4,*]
      Mix0[tmp_lat[ii], *]=Mixps[4,*]
    endfor
  
    ; replace missing Temps with refernce temperature
    ;------------------------------------------------
    dum1=where((finite(Ts) eq 0),c1)
    if(c1 ne 0) then begin
          Ts[dum1]=Ts0[dum1]
      valid1[dum1]=0
    endif

    ; replace missing moisture with refernce moisture
    ;  avoid the fake big number in upper levels(higher than 150hPa)
    ;------------------------------------------------
    dum2=where((finite(Mixs) eq 0),c2)
    Mixs[*,0:49]=0  
    if(c2 ne 0) then begin
        Mixs[dum2]=Mix0[dum2]
      valid2[dum2]=0
    endif

    ; open output file
    ;-------------------------------------
    outfile=OutPath+gps+'/'+string(year_loop,format='(i4.4)')      $
                   +'/Prf_'+string(year_loop,format='(i4.4)')+'.'  $
                           +string(day_loop, format='(i3.3)')+'.dat'
    print, 'outfile=',outfile
    err=file_test(OutPath,/DIRECTORY,/WRITE)
    if(err ne 1) then begin
     print,' OutPath does not exist or does not have write permission'
     print,' OutPath=',OutPath
     STOP
    endif
    outdir=file_dirname(outfile)
    err=file_test(outdir,/DIRECTORY,/WRITE)
    if(err ne 1 ) then begin
     file_mkdir,outdir
    endif
    openw,5,outfile,ERROR=err
    if(err ne 0) then begin
     print,' ERROR opening output file: err=',err
     print,' outfile=',outfile
     STOP
    endif

    ; save results in output file
    ;-------------------------------------
    for jj=0,(in-1) do begin
      printf,5,yrz[jj],dayz[jj],time[jj],leo_id[jj], $
               gps_id[jj],irsz[jj],laty[jj],lonx[jj],$
      format='(1x,i4,1x,i3,1x,f7.3,3(2x,i2),1x,2(f7.2,2x))'
      for kk=0,(nlev-1) do begin
        printf,5,kk,Ps(kk),Ts[jj,kk],valid1[jj,kk], $
                    Mixs[jj,kk],valid2[jj,kk],      $
        format='(i4, 2x, f10.4, 2(f10.4, 4x,i1))'
      endfor
    endfor
    close,5
    free_lun, 5

    ; goto target for next day
    ;---------------------------
    NEXT_DAY:
  endfor  ;  day_loop=day_start,day_end 
endfor  ;  year_loop=YEAR_BEG,YEAR_END
; 
; End Program
;---------------
end
