pro generate_climatology_anomaly
;=====================================================================
;@***h*  MW_Sounder_FCDR/generate_climatology_anomaly.pro
;
; NAME
;        generate_climatology_anomaly.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the given range of months/years, generate a mean monthly 
;              climatology and save the results. For an atlerante range of 
;              months, read in gridded ASCII data for the specified months,
;              subtract the climatological mean for the data and write out
;              anomaly values.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Gridded calibrated polar orbiter data values are read from 
;              ASCII files:
;
;                $InPath/bin_YYYYMM_{chan}.dat
;
; OUTPUTS:     
;              Climatological mean values are written to an ASCII file with the
;              structure and file format:
;
;                $CoutPath/$ClimatologyName
;
;              Monthly Anomaly values are written to ACSII files with the 
;              structure and file format:
;
;                $AoutPath/tls_anamoly_YYYY_1.0
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                   chan= String indicating the AMSU channel number and whether 
;                         the data has been converted using GPSRO data.
;                             values = {'chan9', 'chan9_converted'}
;  CYEAR_BEG,CMONTH_BEG = The begining year and Month for Climatology calculation
;  CYEAR_END,CMONTH_END = The ending year and Month for Climatology calculation
;    YEAR_BEG,MONTH_BEG = The begining year and Month to process
;    YEAR_END,MONTH_END = The ending year and Month to process
;     InPath            = The root path to the monthly files.
;    CoutPath           = The root path for the ASCII Climatology file 
;    AoutPath           = The root path for the ASCII monthly Anomaly files.
;
;                       Path values should contain a trailing '/'
;
;**************************************************************************
           chan='chan9_converted'
      CYEAR_BEG=2002
     CMONTH_BEG=1
      CYEAR_END=2010
     CMONTH_END=12
       YEAR_BEG=2001
      MONTH_BEG=5
       YEAR_END=2010
      MONTH_END=12
         InPath='../Test-Data/ASCII_tmp/combine/'
       CoutPath='../Test-Data/ASCII_tmp/Climatology/'
ClimatologyName='tls_climatology_1.0'
       AoutPath='../Test-Data/ASCII_tmp/Anomaly/'
;*********************************************************
; End of RUN parameters
;*********************************************************

; Set Lat/Lon resolution, determine 
; grid dimensions and allocate space
;--------------------------------------
degree_inter=[2.5,2.5]

lat_num=180/degree_inter[0]
lon_num=360/degree_inter[1]
year_num=1+(CYEAR_END-CYEAR_BEG)

tbs_monthly=fltarr(lon_num,lat_num,year_num,12)-999.
tbs_climatology=fltarr(lon_num,lat_num,12)-999.

;
; Loop over years for climatological mean
;-------------------------------------------
for year_loop=CYEAR_BEG,CYEAR_END do begin
  year_ind=year_loop-CYEAR_BEG

  monthstart=1
  monthend  =12
  if(year_loop eq CYEAR_BEG) then monthstart=CMONTH_BEG
  if(year_loop eq CYEAR_END) then monthend  =CMONTH_END

  ; Loop over month for this year
  ;--------------------------------
  for month_loop=monthstart,monthend do begin
 
    ; Construct the input file name
    ;---------------------------------
    infile1=InPath+'bin_'+string( year_loop,format='(i4.4)') $
                         +string(month_loop,format='(i2.2)') $
                         +'_'+chan+'.dat'
    print,infile1

    ; open the monthly dataset, if there is 
    ; an error goto the next month
    ;----------------------------------------
    openr,lun1,infile1,err=err,/get_lun
    if(err ne 0) then begin
      goto,NEXT_MONTH0
    endif

    ; Get the number of lines in the file, if there are
    ; not enough, then goto the next month, otherwise 
    ; allocate space.
    ;-------------------------------------------------------
    total_num=file_lines(infile1)-1
    if(total_num eq 0) then begin
      goto,NEXT_MONTH0
    endif
    data_all=fltarr(6,total_num)
    title_line='abc'

    ; Read in data
    ;---------------
    readf,lun1,title_line
    readf,lun1,data_all
    close,lun1
    free_lun,lun1

    ; Store the data according to Lat/Lon
    ;--------------------------------------
    lat=data_all[0,*]
    lon=data_all[1,*]
    Tbs_all=data_all[4,*]

    ilat=0
    lat_loop=-90.
    while(lat_loop le (90.-degree_inter[0])) do begin
      lon_loop = -180.
      ilon=0 
      while(lon_loop le (180.-degree_inter[1])) do begin

        ; get the index for the current Lat/Lon
        ;-----------------------------------------
        latlon_id=where((lat eq lat_loop+degree_inter[0]/2) and $
                        (lon eq lon_loop+degree_inter[1]/2),latlon_num)

        ; Copy values for later use
        ;------------------------------
        if(latlon_num eq 1) then begin
          tbs_monthly[ilon,ilat,year_ind,month_loop-1]=Tbs_all[latlon_id]
        endif
        if(lon_loop eq (180.-degree_inter[1])) then begin
         tbs_monthly(ilon  ,ilat,year_ind,month_loop-1)= $
         tbs_monthly(ilon-1,ilat,year_ind,month_loop-1)
        endif

        ; Increment index
        ;---------------------------
        ilon=ilon+1
        lon_loop=lon_loop+degree_inter[1]

      endwhile ; (lon_loop le (180.-degree_inter[1])) 
      ilat=ilat+1
      lat_loop=lat_loop+degree_inter[0]
    endwhile   ; (lat_loop le (90.-degree_inter[0])) 
    
    ; goto target for next month
    ;-----------------------------
    NEXT_MONTH0:

  endfor ; month_loop=monthstart,monthend
endfor   ;  year_loop=CYEAR_BEG,CYEAR_END

;
; Calculate the monthly Climatology
;-----------------------------------
for month_loop=0,11 do begin
  for ilon=0,(lon_num-1) do begin
    for ilat=0,(lat_num-1) do begin
      valid_id=where((tbs_monthly[ilon,ilat,*,month_loop] gt 0),valid_num)
      if(valid_num eq 9) then begin
       tbs_climatology[ilon,ilat,month_loop]=mean(tbs_monthly[ilon,ilat,*,month_loop])
      endif
    endfor
  endfor
endfor

;
; Write out the Climatology values
;------------------------------------
outfile=CoutPath+ClimatologyName
openw,lun2,outfile,/get_lun

for month_loop=1,12 do begin
  printf,lun2,month_loop
  for ilat=0,(lat_num-1) do begin
    for ilon=0,((lon_num/16)-1) do begin  ;144=9*16
      printf,lun2,tbs_climatology[(0+ilon*16):(15+ilon*16),ilat,month_loop-1], $
      format='(16(f8.3,2x))'
    endfor ;lon
  endfor ;lat
endfor ;month_loop

close,lun2
free_lun,lun2

;
; Now, loop through data again and calculate Anomaly values
;-------------------------------------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  monthstart=1
  monthend  =12
  if(year_loop eq YEAR_BEG) then monthstart=MONTH_BEG
  if(year_loop eq YEAR_END) then monthend  =MONTH_END
 
  tbs_anomaly=fltarr(lon_num,lat_num,12)-999.
  title_line_all=strarr(12)

  ; Loop over months for this year
  ;--------------------------------
  for month_loop=monthstart,monthend do begin
 
    ; Construct the input file name
    ;---------------------------------
    infile1=InPath+'bin_'+string( year_loop,format='(i4.4)') $
                         +string(month_loop,format='(i2.2)') $
                         +'_'+chan+'.dat'
    print,infile1

    ; open the monthly dataset, if there is 
    ; an error goto the next month
    ;----------------------------------------
    openr,lun1,infile1,err=err,/get_lun
    if(err ne 0) then begin
      goto,NEXT_MONTH1
    endif

    ; Get the number of lines in the file, if there are
    ; not enough, then goto the next month, otherwise 
    ; allocate space.
    ;-------------------------------------------------------
    total_num=file_lines(infile1)-1
    if(total_num eq 0) then begin
      goto,NEXT_MONTH1
    endif
    data_all=fltarr(6,total_num)
    title_line='abc'

    ; Read in data
    ;---------------
    readf,lun1,title_line
    readf,lun1,data_all
    close,lun1
    free_lun,lun1

    ; Store the data according to Lat/Lon
    ;-------------------------------------
    title_line_all[month_loop-1]=title_line
    lat=data_all[0,*]
    lon=data_all[1,*]
    Tbs_all=data_all[4,*]

    ilat=0
    lat_loop=-90.
    while(lat_loop le (90.-degree_inter[0])) do begin
      lon_loop=-180.
      ilon=0
      while(lon_loop le (180.-degree_inter[1])) do begin

        ; get index for current Lat/Lon
        ;--------------------------------
        latlon_id=where((lat eq lat_loop+degree_inter[0]/2) and $
                        (lon eq lon_loop+degree_inter[1]/2),latlon_num)

        ; Copy values for later use
        ;----------------------------
        if(latlon_num eq 1) then begin
          tbs_anomaly[ilon,ilat,month_loop-1]=Tbs_all[latlon_id]
        endif
        if(lon_loop eq (180.-degree_inter[1])) then begin
          tbs_anomaly[ilon,ilat,month_loop-1]=tbs_anomaly_map[ilon-1,ilat,month_loop-1]
        endif

        ; Increment index
        ;-----------------
        ilon=ilon+1
        lon_loop=lon_loop+degree_inter[1]

      endwhile ; (lon_loop le (180.-degree_inter[1])) 
      ilat=ilat+1
      lat_loop=lat_loop+degree_inter[0]
    endwhile   ; (lat_loop le (90.-degree_inter[0]))
    
    ; goto target for next month
    ;------------------------------
    NEXT_MONTH1:

  endfor  ; month_loop=monthstart,monthend 

  ;
  ; Subtract the Climatology from the data for this year
  ;-------------------------------------------------------
  for month_loop=0,11 do begin
    for ilon=0,(lon_num-1) do begin
      for ilat=0,(lat_num-1) do begin
        if((    tbs_anomaly[ilon,ilat,month_loop] gt 0) and $
           (tbs_climatology[ilon,ilat,month_loop] gt 0)     ) then begin
          tbs_anomaly[ilon,ilat,month_loop]=tbs_anomaly[ilon,ilat,month_loop] $
                                       -tbs_climatology[ilon,ilat,month_loop]
        endif
      endfor
    endfor
  endfor

  ; Open output file for yearly anmonaly data
  ;-------------------------------------------
  outfile=AoutPath+'tls_anamoly_'+string(year_loop,format='(i4.4)')+'_1.0'    
  openw,lun3,outfile,/get_lun

  for month_loop=monthstart,monthend do begin
    printf,lun3,year_loop, month_loop
    printf,lun3,title_line_all[month_loop-1]
    for ilat=0,(lat_num-1) do begin
      for ilon=0,((lon_num/16)-1) do begin
        printf,lun3,tbs_anomaly[(0+ilon*16):(15+ilon*16),ilat,month_loop-1], $
        format='(16(f8.3,2x))'
      endfor
    endfor
  endfor
  close,lun3
  free_lun,lun3

endfor ; year_loop=YEAR_BEG,YEAR_END
;
; End Program
;---------------
end 

