pro match_gps_aqua
;=====================================================================
;@***h*  MW_Sounder_FCDR/match_gps_aqua.pro
;
; NAME
;        match_gps_aqua.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the given range of Julian days, read in AMSU data
;              extracted from the AQUA dataset and simulated
;              AMSU brightness temperatures from profiles of the 
;              specified GPSRO mission.
;
;              Find colocated measurements satisfying the matchup
;              criteria and then save the matched values in an ASCCI
;              file for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              AQUA AMSU data should be stored with the following directory
;              structure and file format:
;
;                $InPath_noaa/aqua/YYYY/amsu_aqua_YYYY.DDD.dat 
;
;              GPSRO simulated AMSU data should be stored in the format:
;
;                $InPath_gps/{gps}/YYYY/AMSU_{gps}_atmBt_YYYY.DDD.dat 
;
; OUTPUTS:     
;              Matched AMSU values are stored in ASCII files with the following
;              directry structure and file format:
;
;                $OutPath/aqua_{gps}/YYYY/match_{gps}_aqua_YYYY.DDD.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                 gps = The name of the GPSRO mission to process.
;                           values = {'champ', 'cosmic', 'grace', 'metopa'}
;   YEAR_BEG,JDAY_BEG = The begining year and Julian day to process
;   YEAR_END,JDAY_END = The ending year and Julian day to process
;   InPath_gps        = The root path to the AMSU Brigtness Temps from GPSRO files.
;   InPath_aqua       = The root path to the AQUA AMSU Brightness Temps files.
;   OutPath           = The root path for the ASCII files to be generated.
;
;                       Path values should contain a trailing '/'
;
; Matching criteria for colocating GPSRO and NOAA data:
;
;        CRI_time = TIME Criteria
;        CRI_dist = DISTANCE Criteria
;        CRI_sang = SATELLITE VIEWING ANGLE Criteria
;
;**************************************************************************
        gps='cosmic'
   YEAR_BEG=2006
   JDAY_BEG=305
   YEAR_END=2006
   JDAY_END=365
 InPath_gps='../Test-Data/ASCII_tmp/gps_AMSU_Tbs/'
InPath_aqua='../Test-Data/ASCII_tmp/extract/'
    OutPath='../Test-Data/ASCII_tmp/match/'
CRI_time=1800. ;                    TIME Criteria units = sec
CRI_dist=0.5   ;                DISTANCE Criteria units = 100km
CRI_sang=15.   ; SATELLITE VIEWING ANGLE Criteria units = angle
  CRI_bt=50.   ;
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Loop over the years to process
;----------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the Julian day for the begining of each month,
  ; the number of days in the month, and the number 
  ; of Julian days for this year
  ;--------------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
    data_begin=[1,32,61,92,122,153,183,214,245,275,306,336]
    data_longs=[31,29,31,30,31,30,31,31,30,31,30,31]
    NJdays=366
  endif else begin
    data_begin=[1,32,60,91,121,152,182,213,244,274,305,335]
    data_longs=[31,28,31,30,31,30,31,31,30,31,30,31]
    NJdays=365
  endelse

  ; Set the starting and ending Julian days
  ;------------------------------------------
  day_start=1
  day_end  =NJdays
  if(year_loop eq YEAR_BEG) then day_start=JDAY_BEG
  if(year_loop eq YEAR_END) then day_end  =JDAY_END

  ; Loop over the Julian days for this year
  ;-------------------------------------------
  for day_loop=day_start,day_end do begin
    ncount = 0L

    ; Open output file
    ;-----------------------------
    outfile=OutPath+'aqua_'+gps+'/'+string(year_loop,format='(i4.4)')     $
            +'/match_'+gps+'_aqua_'+string(year_loop,format='(i4.4)')+'.' $
                                   +string( day_loop,format='(i3.3)')+'.dat'
    err=file_test(OutPath,/DIRECTORY,/WRITE)
    if(err ne 1) then begin
     print,' OutPath does not exist or does not have write permission'
     print,' OutPath=',OutPath
     STOP
    endif
    outdir=file_dirname(outfile)
    err=file_test(outdir,/DIRECTORY,/WRITE)
    if(err ne 1 ) then begin
     file_mkdir,outdir
    endif
    openw,7,outfile,ERROR=err
    if(err ne 0) then begin
     print,' ERROR opening output file: err=',err
     print,' outfile=',outfile
     STOP
    endif

    ; Construct GPS Tbs input file name
    ;---------------------------------------
    infile_gps=InPath_gps+gps+'/'+string(year_loop,format='(i4.4)')     $
          +'/AMSU_'+gps+'_atmBt_'+string(year_loop,format='(i4.4)')+'.' $
                                 +string( day_loop,format='(i3.3)')+'.dat'
    print,infile_gps

    ; Open the GPS dataset for this day and read in data
    ;---------------------------------------------------------
    openr,5,infile_gps,error = err
    if(err ne 0) then begin
      print,'there is no ',gps,' datafile in',year_loop,day_loop
      close,5
      free_lun,5
      goto,NEXT_DAY
    endif else begin
      inum1=file_lines(infile_gps)
      ; If there are not enough profiles, then goto the 
      ; next day, otherwise allocate space and read in data.
      ;-----------------------------------------------------
      if(inum1 lt 2) then begin
        print,'there is no enough ',gps,' datafile in', day_loop
        goto,NEXT_DAY
      endif
      temp=fltarr(23,inum1)
      readf,5,temp
      close,5
      free_lun,5
    endelse

    ; Extract the desired values
    ;-------------------------------
    time_cosmicpp=temp[2,*]*3600.
    lat_cosmicpp =temp[3,*]
    lon_cosmicpp =temp[4,*]

    bt_ch7_cosmicpp =temp[11,*] ;chan_7
    bt_ch8_cosmicpp =temp[12,*] ;chan_8
    bt_ch9_cosmicpp =temp[13,*] ;chan_9
    bt_ch10_cosmicpp=temp[14,*] ;chan_10
    bt_ch11_cosmicpp=temp[15,*] ;chan_11
    bt_ch12_cosmicpp=temp[16,*] ;chan_12

    leo_id=temp[20,*]
    gps_id=temp[21,*]
    irsz  =temp[22,*]
 
    ; read in aqua AMSU data 
    ;---------------------------
    infile_aqua=InPath_aqua+'aqua/'+string( year_loop,format='(i4.4)')     $
                     +'/amsu_aqua_'+string( year_loop,format='(i4.4)')+'.' $
                                   +string(  day_loop,format='(i3.3)')+'.dat'
    print,infile_aqua

    ; Open the AMSU dataset for this day and read in data
    ;---------------------------------------------------------
    openr, 6,infile_aqua,error=err
    if(err ne 0) then begin
      print,'there is no aqua data in',year_loop,day_loop
      close,6
      free_lun,6
      goto,NEXT_DAY
    endif else begin
      ; If there are not enough profiles, then goto the 
      ; next day, otherwise allocate space and read in data.
      ;-------------------------------------------------------
      inum2=file_lines(infile_aqua)-1
      if(inum2 le 1)then begin
        print,'there is no aqua data in', year_loop,day_loop
        goto,NEXT_DAY
      endif
      temp=fltarr(25,inum2)
      readf,6,inum_nouse
      readf,6,temp
      close,6
      free_lun,6
    endelse

    ; Extract the desired values
    ;------------------------------
    dd_aqua  =data_begin[temp[1,*]-1]+temp[2,*]-1
    time_aqua=temp[3,*]*3600.+temp[4,*]*60.+temp[5,*]
    lat_aqua =temp[6,*]
    lon_aqua =temp[7,*]
    sat_aqua =temp[8,*]  ;satellite zenith angle 
    sza_aqua =temp[9,*]  ;solar zenith angle
    bt_ch5_aqua =temp[14,*] ; chan5
    bt_ch6_aqua =temp[15,*] ; chan6
    bt_ch7_aqua =temp[16,*] ; chan7
    bt_ch8_aqua =temp[17,*] ; chan8
    bt_ch9_aqua =temp[18,*] ; chan9
    bt_ch10_aqua=temp[19,*] ; chan10
    bt_ch11_aqua=temp[20,*] ; chan11
    bt_ch12_aqua=temp[21,*] ; chan12

    ; Loop over the number of GPS values
    ;------------------------------------
    daycount=0L
    for kk=0,(inum1-1) do begin ;inum1 is along gps

      ; Load indices where the matchup criteria are satisfied
      ;-------------------------------------------------------
      dist12=sqrt( (abs(lat_aqua-lat_cosmicpp[kk])                               )^2 $
                  +(abs(lon_aqua-lon_cosmicpp[kk])*cos(lat_cosmicpp[kk]*!pi/180.))^2 )
      id=where((    dd_aqua                          eq day_loop) and $
               (abs(time_aqua- time_cosmicpp[kk])    lt CRI_time) and $
               (   (dist12/0.9)                      lt CRI_dist) and $
               (abs(sat_aqua)                        le CRI_sang) and $
               (abs(bt_ch9_aqua-bt_ch9_cosmicpp[kk]) lt CRI_bt  )     ,icount)

      ; Write matched results to output file
      ;---------------------------------------
      if(icount gt 0) then begin
        for k2=0,(icount-1) do begin
          ncount=ncount + 1L
          daycount=daycount + 1L
          printf,7,year_loop,day_loop,time_aqua(id(k2))/3600.,time_cosmicpp[kk]/3600.,$
                   lat_aqua[id[k2]],lon_aqua[id[k2]],$
                   lat_cosmicpp[kk],lon_cosmicpp[kk],$
                   sza_aqua[id[k2]],sat_aqua[id[k2]],$
                   bt_ch7_aqua[ id[k2]],bt_ch7_cosmicpp[ kk],$
                   bt_ch8_aqua[ id[k2]],bt_ch8_cosmicpp[ kk],$
                   bt_ch9_aqua[ id[k2]],bt_ch9_cosmicpp[ kk],$
                   bt_ch10_aqua[id[k2]],bt_ch10_cosmicpp[kk],$
                   bt_ch11_aqua[id[k2]],bt_ch11_cosmicpp[kk],$
                   bt_ch12_aqua[id[k2]],bt_ch12_cosmicpp[kk],$
          format='(i4,1x,i3,2(1x,f6.2),4(1x,f8.3),2x,2(1x,f6.2),12(1x,f9.2))'
        endfor ; k2=0,(icount-1)
      endif  ;icount gt 0 
    endfor ;  kk=0,(inum1-1) 
    print, 'Y=',year_loop,' D=',day_loop, $
           ' day_match,total matches:',daycount,ncount
  
    ; goto target for next day
    ;----------------------------
    NEXT_DAY:

    ; Close output file
    ;-------------------
    close,7
    free_lun,7
  endfor  ; day_loop=day_start,day_end
endfor ;  year_loop=YEAR_BEG,YEAR_END
;
; End Program
;--------------
end 
