pro match_gps_noaa
;=====================================================================
;@***h*  MW_Sounder_FCDR/match_gps_noaa.pro
;
; NAME
;        match_gps_noaa.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For the given range of Julian days, read in AMSU data
;              extracted from the specified NOAA dataset and simulated
;              AMSU brightness temperatures from profiles of the 
;              specified GPSRO mission.
;
;              Find colocated measurements satisfying the matchup
;              criteria and then save the matched values in an ASCCI
;              file for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              NOAA AMSU data should be stored with the following directory
;              structure and file format:
;
;                $InPath_noaa/{noaa}/YYYY/amsu_{noaa}_YYYY.DDD_ch5-12.dat 
;
;              GPSRO simulated AMSU data should be stored in the format:
;
;                $InPath_gps/{gps}/YYYY/AMSU_{gps}_atmBt_YYYY.DDD.dat 
;
; OUTPUTS:     
;              Matched AMSU values are stored in ASCII files with the following
;              directry structure and file format:
;
;                $OutPath/{noaa}_{gps}/YYYY/match_{gps}_{noaa}_YYYY.DDD.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;                 gps = The name of the GPSRO mission to process.
;                           values = {'champ', 'cosmic', 'grace', 'metopa'}
;                noaa = The name of the NOAA satellite to process.
;                           values = {'noaa15', 'noaa16', 'noaa18', 
;                                     'noaa19', 'noaam2'}
;   YEAR_BEG,JDAY_BEG = The begining year and Julian day to process
;   YEAR_END,JDAY_END = The ending year and Julian day to process
;   InPath_gps        = The root path to the AMSU Brigtness Temps from GPSRO files.
;   InPath_noaa       = The root path to the NOAA AMSU Brightness Temps files.
;   OutPath           = The root path for the ASCII files to be generated.
;
;                       Path values should contain a trailing '/'
;
; Matching criteria for colocating GPSRO and NOAA data:
;
;        CRI_time = TIME Criteria
;        CRI_dist = DISTANCE Criteria
;        CRI_sang = SATELLITE VIEWING ANGLE Criteria
;
;**************************************************************************
        gps='cosmic'
       noaa='noaa15'
   YEAR_BEG=2006
   JDAY_BEG=305
   YEAR_END=2006
   JDAY_END=365
 InPath_gps='../Test-Data/ASCII_tmp/gps_AMSU_Tbs/'
InPath_noaa='../Test-Data/ASCII_tmp/extract/'
    OutPath='../Test-Data/ASCII_tmp/match/'
   CRI_time=1800. ;                    TIME Criteria units = sec
   CRI_dist=0.5   ;                DISTANCE Criteria units = 100km
   CRI_sang=15.   ; SATELLITE VIEWING ANGLE Criteria units = angle
;*********************************************************
; End of RUN parameters
;*********************************************************

if(noaa eq 'aqua') then begin
  print,' Cannot match AQUA, use match_gps_aqua.pro'
  STOP
endif

;
; Loop over the years to process
;----------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the Julian day for the begining of each month,
  ; the number of days in the month, and the number 
  ; of Julian days for this year
  ;--------------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
    data_begin=[1,32,61,92,122,153,183,214,245,275,306,336]
    data_longs=[31,29,31,30,31,30,31,31,30,31,30,31]
    NJdays=366
  endif else begin
    data_begin=[1,32,60,91,121,152,182,213,244,274,305,335]
    data_longs=[31,28,31,30,31,30,31,31,30,31,30,31]
    NJdays=365
  endelse

  ; Set the starting and ending Julian days
  ;------------------------------------------
  day_start=1
  day_end  =NJdays
  if(year_loop eq YEAR_BEG) then day_start=JDAY_BEG
  if(year_loop eq YEAR_END) then day_end  =JDAY_END
  
  ; Loop over the Julian days for this year
  ;-------------------------------------------
  for day_loop=day_start,day_end do begin
    ncount = 0L

    ; Open output file
    ;-----------------------
    outfile=OutPath+noaa+'_'+gps+'/'+string(year_loop,format='(i4.4)')     $
         +'/match_'+gps+'_'+noaa+'_'+string(year_loop,format='(i4.4)')+'.' $
                                    +string( day_loop,format='(i3.3)')+'.dat'
    err=file_test(OutPath,/DIRECTORY,/WRITE)
    if(err ne 1) then begin
     print,' OutPath does not exist or does not have write permission'
     print,' OutPath=',OutPath
     STOP
    endif
    outdir=file_dirname(outfile)
    err=file_test(outdir,/DIRECTORY,/WRITE)
    if(err ne 1 ) then begin
     file_mkdir,outdir
    endif
    openw,7,outfile,ERROR=err
    if(err ne 0) then begin
     print,' ERROR opening output file: err=',err
     print,' outfile=',outfile
     STOP
    endif

    ; Construct GPS Tbs input file name
    ;  e.g. 'AMSU_cosmic_Bt_2008_260.dat'
    ;--------------------------------------
    infile_gps=InPath_gps+gps+'/'+string(year_loop,format='(i4.4)')     $
          +'/AMSU_'+gps+'_atmBt_'+string(year_loop,format='(i4.4)')+'.' $
                                 +string( day_loop,format='(i3.3)')+'.dat'

    ; Open the GPS dataset for this day and read in data
    ;---------------------------------------------------------
    openr,5,infile_gps,error=err
    if(err ne 0) then begin
      print,'there is no ',gps,' datafile in', day_loop
      close,5
      free_lun,5
      goto,NEXT_DAY
    endif else begin
      ; If there are not enough profiles, then goto the 
      ; next day, otherwise allocate space and read in data.
      ;-------------------------------------------------------
      inum1=file_lines(infile_gps)
      if(inum1 lt 2)then begin
        print,'there is no enough GPSRO datafile in', day_loop
        goto,NEXT_DAY
      endif
      temp=fltarr(23,inum1)
      readf,5,temp
      close,5
      free_lun,5
    endelse

    ; Extract the desired values
    ;------------------------------
    time_cosmicpp=temp[2,*]*3600
    lat_cosmicpp =temp[3,*]
    lon_cosmicpp =temp[4,*]
   
    bt_ch7_cosmicpp =temp[11,*] ;chan_7
    bt_ch8_cosmicpp =temp[12,*] ;chan_8
    bt_ch9_cosmicpp =temp[13,*] ;chan_9
    bt_ch10_cosmicpp=temp[14,*] ;chan_10
    bt_ch11_cosmicpp=temp[15,*] ;chan_11
    bt_ch12_cosmicpp=temp[16,*] ;chan_12

    leo_id=temp[20,*]
    gps_id=temp[21,*]
    irsz  =temp[22,*]

    ; read in NOAA15 AMSU data 
    ;--------------------------------
    month_loop=min(where(day_loop le (data_begin+data_longs-1)))+1  ;1~12
    infile_noaa=InPath_noaa+noaa+'/'+string( year_loop,format='(i4.4)')     $
                  +'/amsu_'+noaa+'_'+string( year_loop,format='(i4.4)')+'.' $
                                    +string(  day_loop,format='(i3.3)')     $
                                                              +'_ch5-12.dat'

    ; Open the AMSU dataset for this day and read in data
    ;---------------------------------------------------------
    openr,6,infile_noaa,error=err
    if(err ne 0) then begin
      print,'there is no noaa datafile in', day_loop
      close,6
      free_lun,6
      goto,NEXT_DAY
    endif else begin
      ; If there are not enough profiles, then goto the 
      ; next day, otherwise allocate space and read in data.
      ;------------------------------------------------------
      inum2=file_lines(infile_noaa)
      if(Inum2 eq 0) then begin
        print,'there is no noaa data in', day_loop
        goto,NEXT_DAY 
      endif
      temp=fltarr(22,inum2)
      hh_noaa=fltarr(inum2)
      readf,6,temp
      close,6
      free_lun,6
    endelse

    ; Extract the desired values
    ;------------------------------
    dd_noaa  =temp[0,*]
    time_noaa=temp[1,*]  ;unit is second
    iscan    =temp[2,*]
    ispot    =temp[3,*]
    lat_noaa =temp[4,*]
    lon_noaa =temp[5,*]  
    sza_noaa =temp[6,*]  ;solar zenith angle
    sat_noaa =temp[7,*]  ;satellite zenith angle , scaning back and forth + and -
    bt_ch5_noaa =temp[ 8,*] ; chan5
    bt_ch6_noaa =temp[ 9,*] ; chan6
    bt_ch7_noaa =temp[10,*] ; chan7
    bt_ch8_noaa =temp[11,*] ; chan8
    bt_ch9_noaa =temp[12,*] ; chan9
    bt_ch10_noaa=temp[13,*] ; chan10
    bt_ch11_noaa=temp[14,*] ; chan11
    bt_ch12_noaa=temp[15,*] ; chan12
    shelf_a11_noaa=temp[16,*] ; chan12
    shelf_a12_noaa=temp[17,*] ; chan12
    shelf_a2_noaa =temp[18,*] ; chan12
    warm_a11_noaa =temp[19,*] ; chan12
    warm_a12_noaa =temp[20,*] ; chan12
    warm_a2_noaa  =temp[21,*] ; chan12

    ; Loop over the number of GPS values
    ;--------------------------------------
    daycount=0L
    for kk=0,(inum1-1) do begin ;follow the gps number

      ; Load indices where the matchup criteria are satisfied
      ;--------------------------------------------------------
      dist12=sqrt( (abs(lat_noaa-lat_cosmicpp[kk])                               )^2 $
                  +(abs(lon_noaa-lon_cosmicpp[kk])*cos(lat_cosmicpp[kk]*!pi/180.))^2 )
      id=where((    dd_noaa                      eq day_loop) and $
               (abs(time_noaa-time_cosmicpp[kk]) lt CRI_time) and $
               (   (dist12/0.9)                  lt CRI_dist) and $
               (abs(sat_noaa)                    le CRI_sang)     ,icount)
      
      ; Write matched results to output file
      ;------------------------------------------
      if(icount gt 0) then begin
        for k2=0,(icount-1) do begin
          ncount=ncount+1L
          daycount=daycount+1L
          printf,7,year_loop,day_loop,time_noaa[id[k2]]/3600.,time_cosmicpp[kk]/3600.,$
                   lat_noaa[id[k2]],lon_noaa[id[k2]],$
                   lat_cosmicpp[kk],lon_cosmicpp[kk],$
                   sza_noaa[id[k2]],sat_noaa[id[k2]],$
                   bt_ch7_noaa[ id[k2]],bt_ch7_cosmicpp[ kk],$
                   bt_ch8_noaa[ id[k2]],bt_ch8_cosmicpp[ kk],$
                   bt_ch9_noaa[ id[k2]],bt_ch9_cosmicpp[ kk],$
                   bt_ch10_noaa[id[k2]],bt_ch10_cosmicpp[kk],$
                   bt_ch11_noaa[id[k2]],bt_ch11_cosmicpp[kk],$
                   bt_ch12_noaa[id[k2]],bt_ch12_cosmicpp[kk],$
                   shelf_a11_noaa[id[k2]],shelf_a12_noaa[id[k2]],shelf_a2_noaa[id[k2]],$
                   warm_a11_noaa[ id[k2]],warm_a12_noaa[ id[k2]],warm_a2_noaa[ id[k2]],$
          format='(i4,1x,i3,2(1x,f6.2),4(1x,f8.3),2(1x,f6.2),18(1x,f8.3))'
        endfor  ; k2=0,(icount-1)
      endif    ;icount gt 0  
    endfor ; kk=0,(inum1-1) 
    print, 'Y=',year_loop,' D=',day_loop, $
           ' day_match,total matches:',daycount,ncount

    ; goto target for next day
    ;---------------------------
    NEXT_DAY:

    ; Close output file
    ;-------------------
    close,7
    free_lun,7
  endfor ; day_loop=day_start,day_end
endfor ; year_loop=YEAR_BEG,YEAR_END
;
; End Program
;---------------
end 
