pro offset_slope_multigps_aqua_month
;=====================================================================
;@***h*  MW_Sounder_FCDR/offset_slope_multigps_aqua_month.pro
;
; NAME
;        offset_slope_multigps_aqua_month.pro
;
; LOCATION
;        $SRC_DIR/Src_IDL/
;
; DESCRIPTION: 
;              For each month in the specified range, read in the matched
;              AMSU temperatures between the AQUA polar orbiter and
;              for all specified GPSRO missions. Calcualte the linear fit
;              coefficients for each month. Save the resulting monthly 
;              values in an ASCII file for later use.
;
; EXTERNALS:   
;              none.
;
; USAGE:       
;              The RUN parameters specifying the data to process are set 
;              directly below this header. The user needs to edit these 
;              values to process the desired values and then execute the 
;              program from within IDL.
;
; INPUTS:      
;              Matched AMSU values are read from the ASCII files with the
;              following directory structure and file format:
;
;                $InPath/aqua_{gps[*]}/YYYY/match_{gps[*]}_aqua_YYYY.DDD.dat
;
; OUTPUTS:     
;              Linear fit coefficients are written to the ASCII file:
;
;                $OutPath/timeseries_offset_slope_aqua_gpsro.dat
;
; AUTHOR:      
;              Xinjia Zhou / Ben Ho (?)
;
; REVISIONS:   
;              Patrick Callaghan (Formatted for CDR)
;
;
; COPYRIGHT
;              © University Corporation for Atmospheric Research (UCAR) 2011.
;              All rights reserved. Please refer to the UCAR copyright 
;              statement in the main directory
;
;@*****
;=====================================================================


;**************************************************************************
; Set RUN parameters here:
;               gps[] = The array of GPSRO mission names to use
;                           values = {'champ', 'cosmic', 'grace', 'metopa']
;  YEAR_BEG,MONTH_BEG = The begining year and Month to process
;  YEAR_END,MONTH_END = The ending year and Month to process
;   InPath            = The root path to the NOAA-GPSRO match files.
;   OutPath           = The root path for the ASCII slope and offset 
;                       files to be generated.
;
;                       Path values should contain a trailing '/'
;
; Matching criteria for colocating GPSRO and NOAA data, these values can 
; strengthen the Critea that were used when generating the match files.
;
;        CRI_time = TIME Criteria
;        CRI_dist = DISTANCE Criteria
;        CRI_sang = SATELLITE VIEWING ANGLE Criteria
;        CRI_bt   = BRIGHTNESS TEMPERATURE Criteria
;
;**************************************************************************
      gps =['cosmic','champ']
 YEAR_BEG=2006
MONTH_BEG=11
 YEAR_END=2006
MONTH_END=12
   InPath='../Test-Data/ASCII_tmp/match/'
  OutPath='../Test-Data/ASCII_tmp/offset_slope/'
 CRI_time=0.5  ;                    TIME Criteria units = hours
 CRI_dist=0.5  ;                DISTANCE Criteria units = 100km
 CRI_sang=10.  ; SATELLITE VIEWING ANGLE Criteria units = angle
   CRI_bt=10.  ;  BRIGHTNESS TEMPERATURE Criteria units = degrees
;*********************************************************
; End of RUN parameters
;*********************************************************

;
; Construct the output file name and open it
;---------------------------------------------
outfile=OutPath+'timeseries_offset_slope_aqua_gpsro.dat'
err=file_test(OutPath,/DIRECTORY,/WRITE)
if(err ne 1) then begin
  print,' OutPath does not exist or does not have write permission'
  print,' OutPath=',OutPath
  STOP
endif
outdir=file_dirname(outfile)
err=file_test(outdir,/DIRECTORY,/WRITE)
if(err ne 1 ) then begin
  file_mkdir,outdir
endif
openw,lun2,outfile,ERROR=err,/get_lun
if(err ne 0) then begin
  print,' ERROR opening output file: err=',err
  print,' outfile=',outfile
  STOP
endif


MAX_NUM=200000L
;
; Loop over the years to process
;---------------------------------
for year_loop=YEAR_BEG,YEAR_END do begin

  ; Set the starting end ending month for the current year
  ;---------------------------------------------------------
  monthstart=1
  monthend  =12
  if(year_loop eq YEAR_BEG) then monthstart=MONTH_BEG
  if(year_loop eq YEAR_END) then   monthend=MONTH_END

  ;  Set the Julian day for the begining of each month
  ; and the number of days in the month
  ;--------------------------------------------------
  if((year_loop eq 2004) or $
     (year_loop eq 2008) or $
     (year_loop eq 2012)    ) then begin
    month_long= [31,29,31,30,31,30,31,31,30,31,30,31]
    month_start=[1,32,61,92,122,153,183,214,245,275,306,336]
  endif else begin
    month_long= [31,28,31,30,31,30,31,31,30,31,30,31]
    month_start=[1,32,60,91,121,152,182,213,244,274,305,335]
  endelse

  ; Loop over months
  ;-------------------
  for month_loop=monthstart,monthend do begin

    ; Allocate space/variables
    ;--------------------------
    day       =intarr(MAX_NUM)
    time_noaa =fltarr(MAX_NUM)
    time_gps  =fltarr(MAX_NUM)
    lat_noaa  =fltarr(MAX_NUM)
    lon_noaa  =fltarr(MAX_NUM)
    lat_gps   =fltarr(MAX_NUM)
    lon_gps   =fltarr(MAX_NUM)
    ratp      =intarr(MAX_NUM)
    sza       =fltarr(MAX_NUM)
    sat       =fltarr(MAX_NUM)
    bt_noaa   =fltarr(MAX_NUM,6)
    bt_gps    =fltarr(MAX_NUM,6)
    valid_max =intarr(MAX_NUM,6)
    percentage=fltarr(MAX_NUM,6)
    inum=0L

    ; Loop over the days in the current month
    ;-------------------------------------------
    jbeg=month_start[month_loop-1]
    jend=month_start[month_loop-1]+month_long[month_loop-1]-1
    for day_loop=jbeg,jend do begin

      ; Loop over GPSRO missions
      ;---------------------------
      for gps_id=0,(n_elements(gps)-1) do begin

        ; Construct the input file name
        ;-------------------------------
        infile=InPath+'aqua_'+gps[gps_id]+'/'                       $
                             +string(year_loop,format='(i4.4)')     $
                             +'/match_'+gps[gps_id]+'_aqua_'        $
                             +string(year_loop,format='(i4.4)')+'.' $
                             +string( day_loop,format='(i3.3)')+'.dat'
        print,infile

        ; Open the GPS dataset for this day and read in data
        ;---------------------------------------------------------
        openr,lun1,infile,ERROR=err,/get_lun
        if(err ne 0) then begin
          print,'open file err',infile
          goto,NEXT_GPS
        endif else begin
          ; If there are not enough profiles, then goto the next 
          ; GPS mission, otherwise allocate space and read in data. 
          ;----------------------------------------------------------
          total_num=file_lines(infile)
          if(total_num eq 0) then begin
            print,'total_num eq 0',infile
            goto,NEXT_GPS
          endif
          data_total=fltarr(22,total_num)
          readf,lun1,data_total
          close,lun1
          free_lun,lun1
        endelse

        ; Extract the desired values
        ;------------------------------
        day      [inum:(inum+total_num-1)]  =data_total[ 1,*]
        time_noaa[inum:(inum+total_num-1)]  =data_total[ 2,*]
        time_gps [inum:(inum+total_num-1)]  =data_total[ 3,*]
        lat_noaa [inum:(inum+total_num-1)]  =data_total[ 4,*]
        lon_noaa [inum:(inum+total_num-1)]  =data_total[ 5,*]
        lat_gps  [inum:(inum+total_num-1)]  =data_total[ 6,*]
        lon_gps  [inum:(inum+total_num-1)]  =data_total[ 7,*]
        sza      [inum:(inum+total_num-1)]  =data_total[ 8,*]
        sat      [inum:(inum+total_num-1)]  =data_total[ 9,*]
        bt_noaa  [inum:(inum+total_num-1),0]=data_total[10,*] ;chan7
        bt_gps   [inum:(inum+total_num-1),0]=data_total[11,*]
        bt_noaa  [inum:(inum+total_num-1),1]=data_total[12,*] ;8
        bt_gps   [inum:(inum+total_num-1),1]=data_total[13,*]
        bt_noaa  [inum:(inum+total_num-1),2]=data_total[14,*] ;9
        bt_gps   [inum:(inum+total_num-1),2]=data_total[15,*]
        bt_noaa  [inum:(inum+total_num-1),3]=data_total[16,*] ;10
        bt_gps   [inum:(inum+total_num-1),3]=data_total[17,*]
        bt_noaa  [inum:(inum+total_num-1),4]=data_total[18,*] ;11
        bt_gps   [inum:(inum+total_num-1),4]=data_total[19,*]
        bt_noaa  [inum:(inum+total_num-1),5]=data_total[20,*] ;12
        bt_gps   [inum:(inum+total_num-1),5]=data_total[21,*]

        ; Increment the number of matches
        ;---------------------------------
        inum=inum+long(total_num)
        print,'inum=',inum

        ; goto target for the next GPS mission
        ;---------------------------------------
        NEXT_GPS:
      endfor ;  gps_id=0,(n_elements(gps)-1)
    endfor ;  day_loop=jbeg,jend 

    ; If there are not enough matches, goto the next month
    ;------------------------------------------------------
    if(inum lt 5) then goto,NEXT_MONTH

    ; Trim array sizes
    ;------------------
    day      =      day[0:(inum-1)]
    time_noaa=time_noaa[0:(inum-1)]
    time_gps = time_gps[0:(inum-1)]
    lat_noaa = lat_noaa[0:(inum-1)]
    lon_noaa = lon_noaa[0:(inum-1)]
    lat_gps  =  lat_gps[0:(inum-1)]
    lon_gps  =  lon_gps[0:(inum-1)]
    sza      =      sza[0:(inum-1)]
    sat      =      sat[0:(inum-1)]
    bt_noaa  =  bt_noaa[0:(inum-1),*]
    bt_gps   =   bt_gps[0:(inum-1),*]

    ; Allocate some space
    ;-----------------------
       coeff=fltarr(2,6)
    num_chan=intarr(6)

    ; Loop over channels... only CH09 for now
    ;------------------------------------------
    for chan=9,9 do begin

      ; Get indices of data points which 
      ; satisify the given criteria
      ;------------------------------------
      dif_time=abs(time_noaa-time_gps)
      dif_BT  =abs(bt_noaa[*,(chan-7)]-bt_gps[*,(chan-7)])
      distance=sqrt( (abs(lat_noaa-lat_gps)                       )^2  $
                    +(abs(lon_noaa-lon_gps)*cos(lat_noaa*!pi/180.))^2)/0.9

      id=where((  dif_BT lt CRI_bt  ) and $
               (dif_time lt CRI_time) and $
               (distance lt CRI_dist) and $
               (abs(sat) lt CRI_sang),ncount)
      print,'ncount=', ncount 

      ; If there are enough points, calculate 
      ; the slope and offset values.
      ;----------------------------------------
      if(ncount lt 2) then begin
        coeff    =[-999.,-999.]
        mean_chan=-999.
        std_chan =-999.
      endif else begin
        abt      =bt_noaa[id,(chan-7)]
        bbt      =bt_gps [id,(chan-7)]
        coeff    =poly_fit(abt,bbt,1,yfit)
        mean_chan=mean(abt-bbt)
        std_chan =stddev(abt-bbt)
      endelse
      num_chan[chan-7]=ncount

      ; Save the results in the output file
      ;--------------------------------------
      printf,lun2,year_loop,month_loop,coeff,mean_chan,std_chan,ncount,$
      format='(i4,1x,i2,1x,4(f10.3,1x),(i7,1x))'
    endfor ;  chan=9,9 do begin

    ; goto target for next month
    ;-----------------------------
    NEXT_MONTH:
  endfor ;  month_loop=monthstart,monthend
endfor ;  year_loop=YEAR_BEG,YEAR_END
;
; Close Output file
;--------------------
close,lun2
free_lun,lun2
;
; End Program
;---------------
end 
