      program amsu_gps
!=====================================================================
!@***h*  MW_Sounder_FCDR/amsu_gps.f
!
! NAME:        
!   amsu_gps.exe
!
! LOCATION:    
!   $SRC_DIR/Src_fortran/
!
! DESCRIPTION: 
!              For the specified GPSRO mission, this program loops over
!              the specified range of julian days and years, and reads
!              in all of the GPSRO profiles for each day.
!              Each profile is passed to an AMSU forward model routine
!              which calculates the brightness temperatures for 15
!              AMSU channels. The resulting brightness temperatures for
!              each day are written to an ASCII file for later use.
!
! EXTERNALS:   
!              AMSUfast module for AMSU Forward Model Calculations.
!
! USAGE:       
!              For each GPSRO mission {'champ', 'cosmic', 'grace', 'metopa'},
!              the user needs to first edit the RUN parameters for the program 
!              contained in the file 'CONFIG.amsu_gps' to set the mission 
!              name, the range of days to process and the input/output paths 
!              for the data.
!
!              The program is then compiled to generate an executable:
!
!                   bash> make amsu_gps.exe
!                   bash> vi CONFIG.amsu_gps
!                   bash> ./amsu_gps.exe
!
! PARAMETERS:
!                      gps = Name of GPSRO mission to process
!                 YEAR_BEG = Begining year
!                 JDAY_BEG = Begining julian day
!                 YEAR_END = Ending year
!                 JDAY_END = Ending julian day
!                   InPath = Path to interpolated GPSRO profiles
!                  OutPath = Path to output brightness temperature files
!
! INPUTS:      
!              Input ASCII files containing the {gps} profile data should be 
!              stored with the following directory structure and file format:
!
!                    $InPath/{gps}/YYYY/Prf_YYYY.DDD.dat'
!
! OUTPUTS:     
!              AMSU brightness temperatures are stored in the files:
!
!                    $OutPath/{gps}/YYYY/AMSU_{gps}_atmBt_YYYY.DDD.dat
!
! AUTHOR:      
!              ??
!
! REVISIONS:   
!              Patrick Callaghan 
!               *Cleaned up and Packaged the AMSU Forward model routines 
!                 into module AMSUfast_mod.f for CDR
!               *Cleaned up, added program inputs via CONFIG file, and 
!                 Formatted main program for CDR
!@*****
!=====================================================================
      use AMSUfast

      integer,parameter::NLEV=100

      integer       lens
      character*100 infile,outfile,indir,outdir
      character*4   year_str
      character*3   day_str

      integer IUNIT,OUNIT
      integer sfc_lev
      integer ii,kk,kanal
      integer year_loop,day_loop
      integer Njday

      integer year,day,leo_id,gps_id,irsz,lev
      integer inum1(NLEV),inum2(NLEV)
      real    xlat,xlon,time
      real    pres(NLEV),ta(NLEV),wa(NLEV)
      real    surface_temp,emiss_mw
      real    taut(NLEV)
      real    amsu_tb(15)

      ! Program Inputs
      !------------------
      character*10 gps
      integer      YEAR_END,YEAR_BEG
      integer      JDAY_END,JDAY_BEG
      character*80 InPath,OutPath
      integer      InPlen,OutPlen
      namelist /inputs/ gps,
     &                  YEAR_END,YEAR_BEG,
     &                  JDAY_END,JDAY_BEG,
     &                  InPath,OutPath

!---------------------------------------------------------
! initialization
!---------------------------------------------------------
      ! Set default values
      !--------------------
      gps='NOGPS'
      YEAR_BEG=9999
      YEAR_END=9999
      JDAY_BEG=9999
      JDAY_END=9999
      InPath='NOPATH'
      OutPath='NOPATH'

      ! Read input values from CONFIG file
      !-------------------------------------
      open(unit=14,file='CONFIG.amsu_gps',status='old')
      read(14,inputs)
      close(14)

      ! Check that iniput values were specified in CONFIG file
      !-------------------------------------------------------
      if((    YEAR_BEG.eq.9999    ).or.(YEAR_END.eq.9999).or.
     &   (    JDAY_BEG.eq.9999    ).or.(JDAY_END.eq.9999).or.
     &   (    gps(1:5).eq.'NOGPS' ).or.
     &   ( InPath(1:6).eq.'NOPATH').or.
     &   (OutPath(1:6).eq.'NOPATH')) then
        print *,' INPUT ERROR: amsu_gps.exe '
        print *,' Not all program inputs were specified in CONFIG file:'
        print *,'           gps=',gps
        print *,'      YEAR_BEG=',YEAR_BEG
        print *,'      YEAR_END=',YEAR_END
        print *,'      JDAY_BEG=',JDAY_BEG
        print *,'      JDAY_END=',JDAY_END
        print *,'        InPath=',InPath
        print *,'       OutPath=',OutPath
        stop
      endif

      ! Set string lengths, other fixed paramters
      !--------------------------------------------
      InPlen=len_trim(InPath)
      OutPlen=len_trim(OutPath)
      lens=len_trim(gps)
      IUNIT=15
      OUNIT=16
      sfc_lev=100

!---------------------------------------------------------
! Process Profiles
!---------------------------------------------------------
      ! Loop over the years to process
      !---------------------------------
      do year_loop=YEAR_BEG,YEAR_END

        ! Set the number of Julian days for this year
        !---------------------------------------------
        if((year_loop-2000).eq.(4*((year_loop-2000)/4))) then
          Njday=366
        else
          Njday=365
        endif

        ! Set the range of Julian days for the current year
        !---------------------------------------------------
        day_start=1
        day_end  =Njday
        if(year_loop.eq.YEAR_BEG) day_start=JDAY_BEG
        if(year_loop.eq.YEAR_END) day_end  =JDAY_END

        ! Loop over Julian days
        !------------------------
        do day_loop=day_start,day_end

          ! Open the Input/Output files
          !-----------------------------
          write(year_str,'(i4.4)') year_loop
          write(day_str ,'(i3.3)') day_loop
          infile=InPath(1:InPlen)//gps(1:lens)//'/'//year_str//
     &                       '/Prf_'//year_str//'.'//day_str//'.dat' 

          outfile=OutPath(1:OutPlen)//gps(1:lens)//'/'//year_str//
     &                      '/AMSU_'//gps(1:lens)//'_atmBt_'//
     &                              year_str//'.'//day_str//'.dat'
          print*,"inputfile=",infile
          print*,"outputfile=",outfile
          open(IUNIT,FILE=infile,STATUS='OLD',err=135)
          open(OUNIT,FILE=outfile,STATUS='unknown')

          !-----------------------------------------------------
          ! loop of each profile
          !-----------------------------------------------------
          do while(.true.)      ! end in line2000
            read(IUNIT,*,end=121) year,day,time,leo_id,gps_id,irsz,
     &                            xlat,xlon
            do ii=1,100
              read(IUNIT,*) lev,pres(ii),ta(ii),inum1(ii),
     &                                   wa(ii),inum2(ii)
            end do 

            do ii=1,100
              if(wa(ii).eq.0) wa(ii)=1e-5
            end do

            !--------------------------------------------------
            ! Loop for channel AMSU cahnnel from kanal=1-15  
            !--------------------------------------------------
            theta = 0.
            do kanal = 1,15 
              call aqua_amsu_tran(ta,wa,theta,kanal,taut)
              surface_temp=0.0
              emiss_mw=0.0
              call intrte_mws(taut,ta,surface_temp,emiss_mw,
     &                          sfc_lev,tbbd,tbbr,tbbs,tbbt)

              amsu_tb(kanal)=tbbt
            end do 

            write(OUNIT,2001) year,day,time,xlat,xlon,
     &                        (amsu_tb(kk),kk=1,15),leo_id,gps_id,irsz
 2001       format(1x,i4,1x,i3,1x,f8.3,1x,2(1x,f7.2),
     &                       2x,15(f7.2,1x),3(i4,1x))

 2000     end do

  121     close(IUNIT)              
          close(OUNIT)              

  135     continue
        end do
      end do  
!
! End Program
!---------------
      end 
